@@ -1,20 +1,19 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2006,2007  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/disk.h>
@@ -540,7 +539,7 @@ grub_efidisk_open (const char *name, struct grub_disk *disk)
   /* FIXME: Probably it is better to store the block size in the disk,
      and total sectors should be replaced with total blocks.  */
   grub_dprintf ("efidisk", "m = %p, last block = %llx, block size = %x\n",
-		m, m->last_block, m->block_size);
+		m, (unsigned long long) m->last_block, m->block_size);
   disk->total_sectors = (m->last_block
 			 * (m->block_size >> GRUB_DISK_SECTOR_BITS));
   disk->data = d;
@@ -558,8 +557,8 @@ grub_efidisk_close (struct grub_disk *disk __attribute__ ((unused)))
 }
 
 static grub_err_t
-grub_efidisk_read (struct grub_disk *disk, unsigned long sector,
-		   unsigned long size, char *buf)
+grub_efidisk_read (struct grub_disk *disk, grub_disk_addr_t sector,
+		   grub_size_t size, char *buf)
 {
   /* For now, use the disk io interface rather than the block io's.  */
   struct grub_efidisk_data *d;
@@ -571,10 +570,11 @@ grub_efidisk_read (struct grub_disk *disk, unsigned long sector,
   dio = d->disk_io;
   bio = d->block_io;
 
-  grub_dprintf ("efidisk", "reading 0x%lx sectors at the sector 0x%lx from %s\n",
-		size, sector, disk->name);
+  grub_dprintf ("efidisk",
+		"reading 0x%lx sectors at the sector 0x%llx from %s\n",
+		(unsigned long) size, (unsigned long long) sector, disk->name);
   
-  status = dio->read (dio, bio->media->media_id,
+  status = efi_call_5 (dio->read, dio, bio->media->media_id,
 		      (grub_efi_uint64_t) sector << GRUB_DISK_SECTOR_BITS,
 		      (grub_efi_uintn_t) size << GRUB_DISK_SECTOR_BITS,
 		      buf);
@@ -585,8 +585,8 @@ grub_efidisk_read (struct grub_disk *disk, unsigned long sector,
 }
 
 static grub_err_t
-grub_efidisk_write (struct grub_disk *disk, unsigned long sector,
-		    unsigned long size, const char *buf)
+grub_efidisk_write (struct grub_disk *disk, grub_disk_addr_t sector,
+		    grub_size_t size, const char *buf)
 {
   /* For now, use the disk io interface rather than the block io's.  */
   struct grub_efidisk_data *d;
@@ -598,10 +598,11 @@ grub_efidisk_write (struct grub_disk *disk, unsigned long sector,
   dio = d->disk_io;
   bio = d->block_io;
   
-  grub_dprintf ("efidisk", "writing 0x%lx sectors at the sector 0x%lx to %s\n",
-		size, sector, disk->name);
+  grub_dprintf ("efidisk",
+		"writing 0x%lx sectors at the sector 0x%llx to %s\n",
+		(unsigned long) size, (unsigned long long) sector, disk->name);
   
-  status = dio->write (dio, bio->media->media_id,
+  status = efi_call_5 (dio->write, dio, bio->media->media_id,
 		       (grub_efi_uint64_t) sector << GRUB_DISK_SECTOR_BITS,
 		       (grub_efi_uintn_t) size << GRUB_DISK_SECTOR_BITS,
 		       (void *) buf);
