@@ -1,7 +1,7 @@
 /* cpio.c - cpio and tar filesystem.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2007,2008,2009 Free Software Foundation, Inc.
+ *  Copyright (C) 2007,2008 Free Software Foundation, Inc.
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,10 +23,9 @@
 #include <grub/disk.h>
 #include <grub/dl.h>
 
-#ifndef MODE_USTAR
-/* cpio support */
 #define	MAGIC_BCPIO	070707
-struct head
+
+struct HEAD_BCPIO
 {
   grub_uint16_t magic;
   grub_uint16_t dev;
@@ -42,10 +41,10 @@ struct head
   grub_uint16_t filesize_1;
   grub_uint16_t filesize_2;
 } __attribute__ ((packed));
-#else
-/* tar support */
+
 #define MAGIC_USTAR	"ustar"
-struct head
+
+struct HEAD_USTAR
 {
   char name[100];
   char mode[8];
@@ -64,7 +63,11 @@ struct head
   char devminor[8];
   char prefix[155];
 } __attribute__ ((packed));
-#endif
+
+#define HEAD_LENG	sizeof(struct HEAD_USTAR)
+
+#define MODE_BCPIO	1
+#define MODE_USTAR	2
 
 struct grub_cpio_data
 {
@@ -72,23 +75,27 @@ struct grub_cpio_data
   grub_uint32_t hofs;
   grub_uint32_t dofs;
   grub_uint32_t size;
+  int mode;
 };
 
+#ifndef GRUB_UTIL
 static grub_dl_t my_mod;
+#endif
 
 static grub_err_t
 grub_cpio_find_file (struct grub_cpio_data *data, char **name,
 		     grub_uint32_t * ofs)
 {
-#ifndef MODE_USTAR
-      struct head hd;
+  if (data->mode == MODE_BCPIO)
+    {
+      struct HEAD_BCPIO hd;
 
       if (grub_disk_read
-	  (data->disk, 0, data->hofs, sizeof (hd), &hd))
+	  (data->disk, 0, data->hofs, sizeof (hd), (char *) &hd))
 	return grub_errno;
 
       if (hd.magic != MAGIC_BCPIO)
-	return grub_error (GRUB_ERR_BAD_FS, "invalid cpio archive");
+	return grub_error (GRUB_ERR_BAD_FS, "Invalid cpio archive");
 
       data->size = (((grub_uint32_t) hd.filesize_1) << 16) + hd.filesize_2;
 
@@ -116,11 +123,13 @@ grub_cpio_find_file (struct grub_cpio_data *data, char **name,
       *ofs = data->dofs + data->size;
       if (data->size & 1)
 	(*ofs)++;
-#else
-      struct head hd;
+    }
+  else
+    {
+      struct HEAD_USTAR hd;
 
       if (grub_disk_read
-	  (data->disk, 0, data->hofs, sizeof (hd), &hd))
+	  (data->disk, 0, data->hofs, sizeof (hd), (char *) &hd))
 	return grub_errno;
 
       if (!hd.name[0])
@@ -130,7 +139,7 @@ grub_cpio_find_file (struct grub_cpio_data *data, char **name,
 	}
 
       if (grub_memcmp (hd.magic, MAGIC_USTAR, sizeof (MAGIC_USTAR) - 1))
-	return grub_error (GRUB_ERR_BAD_FS, "invalid tar archive");
+	return grub_error (GRUB_ERR_BAD_FS, "Invalid tar archive");
 
       if ((*name = grub_strdup (hd.name)) == NULL)
 	return grub_errno;
@@ -139,25 +148,26 @@ grub_cpio_find_file (struct grub_cpio_data *data, char **name,
       data->dofs = data->hofs + GRUB_DISK_SECTOR_SIZE;
       *ofs = data->dofs + ((data->size + GRUB_DISK_SECTOR_SIZE - 1) &
 			   ~(GRUB_DISK_SECTOR_SIZE - 1));
-#endif
+    }
   return GRUB_ERR_NONE;
 }
 
 static struct grub_cpio_data *
 grub_cpio_mount (grub_disk_t disk)
 {
-  struct head hd;
+  char hd[HEAD_LENG];
   struct grub_cpio_data *data;
+  int mode;
 
-  if (grub_disk_read (disk, 0, 0, sizeof (hd), &hd))
+  if (grub_disk_read (disk, 0, 0, sizeof (hd), hd))
     goto fail;
 
-#ifndef MODE_USTAR
-  if (hd.magic != MAGIC_BCPIO)
-#else
-  if (grub_memcmp (hd.magic, MAGIC_USTAR,
-		   sizeof (MAGIC_USTAR) - 1))
-#endif
+  if (((struct HEAD_BCPIO *) hd)->magic == MAGIC_BCPIO)
+    mode = MODE_BCPIO;
+  else if (!grub_memcmp (((struct HEAD_USTAR *) hd)->magic, MAGIC_USTAR,
+			 sizeof (MAGIC_USTAR) - 1))
+    mode = MODE_USTAR;
+  else
     goto fail;
 
   data = (struct grub_cpio_data *) grub_malloc (sizeof (*data));
@@ -165,24 +175,18 @@ grub_cpio_mount (grub_disk_t disk)
     goto fail;
 
   data->disk = disk;
+  data->mode = mode;
 
   return data;
 
 fail:
-  grub_error (GRUB_ERR_BAD_FS, "not a "
-#ifdef MODE_USTAR
-	      "tar"
-#else
-	      "cpio"
-#endif
-	      " filesystem");
+  grub_error (GRUB_ERR_BAD_FS, "not a cpio filesystem");
   return 0;
 }
 
 static grub_err_t
 grub_cpio_dir (grub_device_t device, const char *path,
-	       int (*hook) (const char *filename,
-			    const struct grub_dirhook_info *info))
+	       int (*hook) (const char *filename, int dir))
 {
   struct grub_cpio_data *data;
   grub_uint32_t ofs;
@@ -190,7 +194,9 @@ grub_cpio_dir (grub_device_t device, const char *path,
   const char *np;
   int len;
 
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
+#endif
 
   prev = 0;
 
@@ -224,11 +230,7 @@ grub_cpio_dir (grub_device_t device, const char *path,
 
 	  if ((!prev) || (grub_strcmp (prev, name) != 0))
 	    {
-	      struct grub_dirhook_info info;
-	      grub_memset (&info, 0, sizeof (info));
-	      info.dir = (p != NULL);
-
-	      hook (name + len, &info);
+	      hook (name + len, p != NULL);
 	      if (prev)
 		grub_free (prev);
 	      prev = name;
@@ -247,7 +249,9 @@ fail:
   if (data)
     grub_free (data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
@@ -258,9 +262,10 @@ grub_cpio_open (grub_file_t file, const char *name)
   struct grub_cpio_data *data;
   grub_uint32_t ofs;
   char *fn;
-  int i, j;
 
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
+#endif
 
   data = grub_cpio_mount (file->device->disk);
   if (!data)
@@ -278,38 +283,15 @@ grub_cpio_open (grub_file_t file, const char *name)
 	  break;
 	}
 
-      /* Compare NAME and FN by hand in order to cope with duplicate
-	 slashes.  */
-      i = 0;
-      j = 0;
-      while (name[i] == '/')
-	i++;
-      while (1)
+      if (grub_strcmp (name + 1, fn) == 0)
 	{
-	  if (name[i] != fn[j])
-	    goto no_match;
+	  file->data = data;
+	  file->size = data->size;
+	  grub_free (fn);
 
-	  if (name[i] == '\0')
-	    break;
-
-	  while (name[i] == '/' && name[i+1] == '/')
-	    i++;
-
-	  i++;
-	  j++;
+	  return GRUB_ERR_NONE;
 	}
 
-      if (name[i] != fn[j])
-	goto no_match;
-
-      file->data = data;
-      file->size = data->size;
-      grub_free (fn);
-
-      return GRUB_ERR_NONE;
-
-    no_match:
-
       grub_free (fn);
       data->hofs = ofs;
     }
@@ -319,7 +301,9 @@ fail:
   if (data)
     grub_free (data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
@@ -339,38 +323,32 @@ grub_cpio_close (grub_file_t file)
 {
   grub_free (file->data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
 
 static struct grub_fs grub_cpio_fs = {
-#ifdef MODE_USTAR
-  .name = "tarfs",
-#else
   .name = "cpiofs",
-#endif
   .dir = grub_cpio_dir,
   .open = grub_cpio_open,
   .read = grub_cpio_read,
   .close = grub_cpio_close,
+  .label = 0,
+  .next = 0
 };
 
-#ifdef MODE_USTAR
-GRUB_MOD_INIT (tar)
-#else
 GRUB_MOD_INIT (cpio)
-#endif
 {
   grub_fs_register (&grub_cpio_fs);
+#ifndef GRUB_UTIL
   my_mod = mod;
+#endif
 }
 
-#ifdef MODE_USTAR
-GRUB_MOD_FINI (tar)
-#else
 GRUB_MOD_FINI (cpio)
-#endif
 {
   grub_fs_unregister (&grub_cpio_fs);
 }
