@@ -3,26 +3,25 @@
  *  GRUB  --  GRand Unified Bootloader
  *  Copyright (C) 2003,2004,2005,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Magic value used to identify an ext2 filesystem.  */
 #define	EXT2_MAGIC		0xEF53
 /* Amount of indirect blocks in an inode.  */
 #define INDIRECT_BLOCKS		12
-/* Maximum lenght of a pathname.  */
+/* Maximum length of a pathname.  */
 #define EXT2_PATH_MAX		4096
 /* Maximum nesting of symlinks, used to prevent a loop.  */
 #define	EXT2_MAX_SYMLINKCNT	8
@@ -147,7 +146,7 @@ struct grub_ext2_inode
       grub_uint32_t dir_blocks[INDIRECT_BLOCKS];
       grub_uint32_t indir_block;
       grub_uint32_t double_indir_block;
-      grub_uint32_t tripple_indir_block;
+      grub_uint32_t triple_indir_block;
     } blocks;
     char symlink[60];
   };
@@ -252,11 +251,11 @@ grub_ext2_read_block (grub_fshelp_node_t node, int fileblock)
       
       blknr = grub_le_to_cpu32 (indir[rblock % perblock]);
     }
-  /* Tripple indirect.  */
+  /* triple indirect.  */
   else
     {
       grub_error (GRUB_ERR_NOT_IMPLEMENTED_YET,
-		  "ext2fs doesn't support tripple indirect blocks");
+		  "ext2fs doesn't support triple indirect blocks");
       blknr = -1;
     }
 
@@ -268,7 +267,7 @@ grub_ext2_read_block (grub_fshelp_node_t node, int fileblock)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_ext2_read_file (grub_fshelp_node_t node,
-		     void (*read_hook) (grub_disk_addr_t sector,
+		     void NESTED_FUNC_ATTR (*read_hook) (grub_disk_addr_t sector,
 					unsigned offset, unsigned length),
 		     int pos, grub_size_t len, char *buf)
 {
