@@ -2,21 +2,20 @@
    SUSP, Rock Ridge.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2004, 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2004,2005,2006,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/err.h>
@@ -39,6 +38,12 @@
 #define GRUB_ISO9660_RR_DOT		2
 #define GRUB_ISO9660_RR_DOTDOT		4
 
+#define GRUB_ISO9660_VOLDESC_BOOT	0
+#define GRUB_ISO9660_VOLDESC_PRIMARY	1
+#define GRUB_ISO9660_VOLDESC_SUPP	2
+#define GRUB_ISO9660_VOLDESC_PART	3
+#define GRUB_ISO9660_VOLDESC_END	255
+
 /* The head of a volume descriptor.  */
 struct grub_iso9660_voldesc
 {
@@ -68,11 +73,13 @@ struct grub_iso9660_primary_voldesc
   struct grub_iso9660_voldesc voldesc;
   grub_uint8_t unused1[33];
   grub_uint8_t volname[32];
-  grub_uint8_t unused2[60];
+  grub_uint8_t unused2[16];
+  grub_uint8_t escape[32];
+  grub_uint8_t unused3[12];
   grub_uint32_t path_table_size;
-  grub_uint8_t unused3[4];
+  grub_uint8_t unused4[4];
   grub_uint32_t path_table;
-  grub_uint8_t unused4[12];
+  grub_uint8_t unused5[12];
   struct grub_iso9660_dir rootdir;
 } __attribute__ ((packed));
 
@@ -116,6 +123,7 @@ struct grub_iso9660_data
   unsigned int length;
   int rockridge;
   int susp_skip;
+  int joliet;
 };
 
 struct grub_fshelp_node
@@ -169,11 +177,11 @@ grub_iso9660_susp_iterate (struct grub_iso9660_data *data,
 	 ((char *) entry + entry->len))
     {
       /* The last entry.  */
-      if (!grub_strncmp (entry->sig, "ST", 2))
+      if (grub_strncmp ((char *) entry->sig, "ST", 2) == 0)
 	break;
       
       /* Additional entries are stored elsewhere.  */
-      if (!grub_strncmp (entry->sig, "CE", 2))
+      if (grub_strncmp ((char *) entry->sig, "CE", 2) == 0)
 	{
 	  struct grub_iso9660_susp_ce *ce;
 	  
@@ -198,6 +206,23 @@ grub_iso9660_susp_iterate (struct grub_iso9660_data *data,
   return 0;
 }
 
+static char *
+grub_iso9660_convert_string (grub_uint16_t *us, int len)
+{
+  char *p;
+  int i;
+
+  p = grub_malloc (len * 4 + 1);
+  if (! p)
+    return p;
+
+  for (i=0; i<len; i++)
+    us[i] = grub_be_to_cpu16 (us[i]);
+
+  *grub_utf16_to_utf8 ((grub_uint8_t *) p, us, len) = '\0';
+
+  return p;
+}
 
 static struct grub_iso9660_data *
 grub_iso9660_mount (grub_disk_t disk)
@@ -208,6 +233,8 @@ grub_iso9660_mount (grub_disk_t disk)
   int sua_size;
   char *sua;
   struct grub_iso9660_susp_entry *entry;
+  struct grub_iso9660_primary_voldesc voldesc;
+  int block;
   
   auto grub_err_t susp_iterate (struct grub_iso9660_susp_entry *);
   
@@ -215,7 +242,7 @@ grub_iso9660_mount (grub_disk_t disk)
     {
       /* The "ER" entry is used to detect extensions.  The
 	 `IEEE_P1285' extension means Rock ridge.  */
-      if (!grub_strncmp (susp_entry->sig, "ER", 2))
+      if (grub_strncmp ((char *) susp_entry->sig, "ER", 2) == 0)
 	{
 	  data->rockridge = 1;
 	  return 1;
@@ -224,27 +251,52 @@ grub_iso9660_mount (grub_disk_t disk)
     }
   
   data = grub_malloc (sizeof (struct grub_iso9660_data));
-  if (!data)
+  if (! data)
     return 0;
   
-  /* Read the superblock.  */
-  if (grub_disk_read (disk, 16 << GRUB_ISO9660_LOG2_BLKSZ, 0,
-		      sizeof (struct grub_iso9660_primary_voldesc),
-		      (char *) &data->voldesc))
-    {
-      grub_error (GRUB_ERR_BAD_FS, "not a iso9660 filesystem");
-      goto fail;
-    }
-
-  if (grub_strncmp (data->voldesc.voldesc.magic, "CD001", 5))
-    {
-      grub_error (GRUB_ERR_BAD_FS, "not a iso9660 filesystem");
-      goto fail;
-    }
-  
   data->disk = disk;
   data->rockridge = 0;
-  
+  data->joliet = 0;
+
+  block = 16;
+  do
+    {
+      int copy_voldesc = 0;
+
+      /* Read the superblock.  */
+      if (grub_disk_read (disk, block << GRUB_ISO9660_LOG2_BLKSZ, 0,
+			  sizeof (struct grub_iso9660_primary_voldesc),
+			  (char *) &voldesc))
+        {
+          grub_error (GRUB_ERR_BAD_FS, "not a iso9660 filesystem");
+          goto fail;
+        }
+
+      if (grub_strncmp ((char *) voldesc.voldesc.magic, "CD001", 5) != 0)
+        {
+          grub_error (GRUB_ERR_BAD_FS, "not a iso9660 filesystem");
+          goto fail;
+        }
+
+      if (voldesc.voldesc.type == GRUB_ISO9660_VOLDESC_PRIMARY)
+        copy_voldesc = 1;
+      else if ((voldesc.voldesc.type == GRUB_ISO9660_VOLDESC_SUPP) &&
+               (voldesc.escape[0] == 0x25) && (voldesc.escape[1] == 0x2f) &&
+               ((voldesc.escape[2] == 0x40) ||	/* UCS-2 Level 1.  */
+                (voldesc.escape[2] == 0x43) ||  /* UCS-2 Level 2.  */
+                (voldesc.escape[2] == 0x45)))	/* UCS-2 Level 3.  */
+        {
+          copy_voldesc = 1;
+          data->joliet = 1;
+        }
+
+      if (copy_voldesc)
+        grub_memcpy((char *) &data->voldesc, (char *) &voldesc,
+                    sizeof (struct grub_iso9660_primary_voldesc));
+
+      block++;
+    } while (voldesc.voldesc.type != GRUB_ISO9660_VOLDESC_END);
+
   /* Read the system use area and test it to see if SUSP is
      supported.  */
   if (grub_disk_read (disk, (grub_le_to_cpu32 (data->voldesc.rootdir.first_sector)
@@ -255,20 +307,12 @@ grub_iso9660_mount (grub_disk_t disk)
       goto fail;
     }
   
-  if (grub_disk_read (disk, (grub_le_to_cpu32 (data->voldesc.rootdir.first_sector)
-			     << GRUB_ISO9660_LOG2_BLKSZ), 0,
-		      sizeof (rootdir), (char *) &rootdir))
-    {
-      grub_error (GRUB_ERR_BAD_FS, "not a iso9660 filesystem");
-      goto fail;
-    }
-  
   sua_pos = (sizeof (rootdir) + rootdir.namelen
 	     + (rootdir.namelen % 2) - 1);
   sua_size = rootdir.len - sua_pos;
 
   sua = grub_malloc (sua_size);
-  if (!sua)
+  if (! sua)
     goto fail;
   
   if (grub_disk_read (disk, (grub_le_to_cpu32 (data->voldesc.rootdir.first_sector)
@@ -282,15 +326,15 @@ grub_iso9660_mount (grub_disk_t disk)
   entry = (struct grub_iso9660_susp_entry *) sua;
   
   /* Test if the SUSP protocol is used on this filesystem.  */
-  if (!grub_strncmp (entry->sig, "SP", 2))
+  if (grub_strncmp ((char *) entry->sig, "SP", 2) == 0)
     {
-      /* The 2nd data byte stored how many bytes are skipped everytime
+      /* The 2nd data byte stored how many bytes are skipped every time
 	 to get to the SUA (System Usage Area).  */
       data->susp_skip = entry->data[2];
       entry = (struct grub_iso9660_susp_entry *) ((char *) entry + entry->len);
       
       /* Iterate over the entries in the SUA area to detect
-	 entensions.  */
+	 extensions.  */
       if (grub_iso9660_susp_iterate (data,
 				     (grub_le_to_cpu32 (data->voldesc.rootdir.first_sector)
 				      << GRUB_ISO9660_LOG2_BLKSZ),
@@ -324,17 +368,16 @@ grub_iso9660_read_symlink (grub_fshelp_node_t node)
       int size = grub_strlen (symlink);
       
       symlink = grub_realloc (symlink, size + len + 1);
-      if (!symlink)
+      if (! symlink)
 	return;
+      
       grub_strncat (symlink, part, len);
-
-      return;
     }
     
   /* Read in a symlink.  */
   grub_err_t susp_iterate_sl (struct grub_iso9660_susp_entry *entry)
     {
-      if (!grub_strncmp ("SL", entry->sig, 2))
+      if (grub_strncmp ("SL", (char *) entry->sig, 2) == 0)
 	{
 	  unsigned int pos = 1;
 
@@ -355,7 +398,7 @@ grub_iso9660_read_symlink (grub_fshelp_node_t node)
 		    /* The data on pos + 2 is the actual data, pos + 1
 		       is the length.  Both are part of the `Component
 		       Record'.  */
-		    add_part (&entry->data[pos + 2],
+		    add_part ((char *) &entry->data[pos + 2],
 			      entry->data[pos + 1]);
 		    if ((entry->data[pos] & 1))
 		      addslash = 1;
@@ -432,7 +475,7 @@ grub_iso9660_iterate_dir (grub_fshelp_node_t dir,
   grub_err_t susp_iterate_dir (struct grub_iso9660_susp_entry *entry)
     {
       /* The filename in the rock ridge entry.  */
-      if (!grub_strncmp ("NM", entry->sig, 2))
+      if (grub_strncmp ("NM", (char *) entry->sig, 2) == 0)
 	{
 	  /* The flags are stored at the data position 0, here the
 	     filename type is stored.  */
@@ -457,12 +500,12 @@ grub_iso9660_iterate_dir (grub_fshelp_node_t dir,
 		  filename[0] = '\0';
 		}
 	      filename_alloc = 1;
-	      grub_strncpy (filename, &entry->data[1], size);
-	      filename [size] = '\0';
+	      grub_strncpy (filename, (char *) &entry->data[1], size);
+	      filename[size] = '\0';
 	    }
 	}
       /* The mode information (st_mode).  */
-      else if (!grub_strncmp (entry->sig, "PX", 2))
+      else if (grub_strncmp ((char *) entry->sig, "PX", 2) == 0)
 	{
 	  /* At position 0 of the PX record the st_mode information is
 	     stored.  */
@@ -574,6 +617,20 @@ grub_iso9660_iterate_dir (grub_fshelp_node_t dir,
 	      filename = name;
 	  }
 	
+        if (dir->data->joliet)
+          {
+            char *oldname;
+
+            oldname = filename;
+            filename = grub_iso9660_convert_string
+                  ((grub_uint16_t *) oldname, dirent.namelen >> 1);
+
+            if (filename_alloc)
+              grub_free (oldname);
+
+            filename_alloc = 1;
+          }
+
 	if (hook (filename, type, node))
 	  {
 	    if (filename_alloc)
@@ -623,7 +680,7 @@ grub_iso9660_dir (grub_device_t device, const char *path,
 #endif
 
   data = grub_iso9660_mount (device->disk);
-  if (!data)
+  if (! data)
     goto fail;
   
   rootnode.data = data;
@@ -704,7 +761,7 @@ grub_iso9660_open (struct grub_file *file, const char *name)
 
 
 static grub_ssize_t
-grub_iso9660_read (grub_file_t file, char *buf, grub_ssize_t len)
+grub_iso9660_read (grub_file_t file, char *buf, grub_size_t len)
 {
   struct grub_iso9660_data *data = 
     (struct grub_iso9660_data *) file->data;
@@ -742,7 +799,11 @@ grub_iso9660_label (grub_device_t device, char **label)
   
   if (data)
     {
-      *label = grub_strndup (data->voldesc.volname, 32);
+      if (data->joliet)
+        *label = grub_iso9660_convert_string
+                 ((grub_uint16_t *) &data->voldesc.volname, 16);
+      else
+        *label = grub_strndup ((char *) data->voldesc.volname, 32);
       grub_free (data);
     }
   else
