@@ -1,21 +1,20 @@
 /* jfs.c - JFS.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2004,2005,2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2004,2005,2006,2007,2008  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/err.h>
@@ -44,14 +43,14 @@ struct grub_jfs_sblock
   grub_uint8_t magic[4];
   grub_uint32_t version;
   grub_uint64_t ag_size;
-  
+
   /* The size of a filesystem block in bytes.  XXX: currently only
      4096 was tested.  */
   grub_uint32_t blksz;
   grub_uint16_t log2_blksz;
-  
+
   grub_uint8_t unused[71];
-  grub_uint8_t volname[11];  
+  grub_uint8_t volname[11];
 };
 
 struct grub_jfs_extent
@@ -59,7 +58,7 @@ struct grub_jfs_extent
   /* The length of the extent in filesystem blocks.  */
   grub_uint16_t length;
   grub_uint8_t length2;
-  
+
   /* The physical offset of the first block on the disk.  */
   grub_uint8_t blk1;
   grub_uint32_t blk2;
@@ -77,10 +76,10 @@ struct grub_jfs_treehead
 {
   grub_uint64_t next;
   grub_uint64_t prev;
-  
-  grub_uint8_t flags; 
+
+  grub_uint8_t flags;
   grub_uint8_t unused;
-  
+
   grub_uint16_t count;
   grub_uint16_t max;
   grub_uint8_t unused2[10];
@@ -95,7 +94,7 @@ struct grub_jfs_tree_extent
   /* The offset is the key used to lookup an extent.  */
   grub_uint8_t offset1;
   grub_uint32_t offset2;
-  
+
   struct grub_jfs_extent extent;
 } __attribute__ ((packed));
 
@@ -106,15 +105,15 @@ struct grub_jfs_tree_dir
      this level.  */
   grub_uint64_t nextb;
   grub_uint64_t prevb;
-  
+
   grub_uint8_t flags;
-  
+
   /* The amount of dirents in this node.  */
   grub_uint8_t count;
   grub_uint8_t freecnt;
   grub_uint8_t freelist;
   grub_uint8_t maxslot;
-  
+
   /* The location of the sorted array of pointers to dirents.  */
   grub_uint8_t sindex;
   grub_uint8_t unused[10];
@@ -162,7 +161,7 @@ struct grub_jfs_inode
   grub_uint32_t mode;
   grub_uint8_t unused3[72];
   grub_uint8_t unused4[96];
-  
+
   union
   {
     /* The tree describing the extents of the file.  */
@@ -178,7 +177,7 @@ struct grub_jfs_inode
       {
 	grub_uint8_t unused[16];
 	grub_uint8_t flags;
-	
+
 	/* Amount of dirents in this node.  */
 	grub_uint8_t count;
 	grub_uint8_t freecnt;
@@ -223,16 +222,14 @@ struct grub_jfs_diropen
   char *sorted;
   struct grub_jfs_leaf_dirent *leaf;
   struct grub_jfs_leaf_next_dirent *next_leaf;
-  
+
   /* The filename and inode of the last read dirent.  */
   char name[255];
   grub_uint32_t ino;
 } __attribute__ ((packed));
 
 
-#ifndef GRUB_UTIL
 static grub_dl_t my_mod;
-#endif
 
 static grub_err_t grub_jfs_lookup_symlink (struct grub_jfs_data *data, int ino);
 
@@ -244,19 +241,19 @@ grub_jfs_blkno (struct grub_jfs_data *data, struct grub_jfs_inode *inode,
 {
   auto int getblk (struct grub_jfs_treehead *treehead,
 		   struct grub_jfs_tree_extent *extents);
-  
+
   int getblk (struct grub_jfs_treehead *treehead,
 	      struct grub_jfs_tree_extent *extents)
     {
       int found = -1;
       int i;
-      
+
       for (i = 0; i < grub_le_to_cpu16 (treehead->count) - 2; i++)
 	{
 	  if (treehead->flags & GRUB_JFS_TREE_LEAF)
 	    {
 	      /* Read the leafnode.  */
-	      if (grub_le_to_cpu32 (extents[i].offset2) <= blk 
+	      if (grub_le_to_cpu32 (extents[i].offset2) <= blk
 		  && ((grub_le_to_cpu16 (extents[i].extent.length))
 		      + (extents[i].extent.length2 << 8)
 		      + grub_le_to_cpu32 (extents[i].offset2)) > blk)
@@ -267,7 +264,7 @@ grub_jfs_blkno (struct grub_jfs_data *data, struct grub_jfs_inode *inode,
 	    if (blk >= grub_le_to_cpu32 (extents[i].offset2))
 	      found = i;
 	}
-      
+
       if (found != -1)
 	{
 	  struct
@@ -275,20 +272,20 @@ grub_jfs_blkno (struct grub_jfs_data *data, struct grub_jfs_inode *inode,
 	    struct grub_jfs_treehead treehead;
 	    struct grub_jfs_tree_extent extents[254];
 	  } tree;
-	  	  
+
 	  if (grub_disk_read (data->disk,
 			      grub_le_to_cpu32 (extents[found].extent.blk2)
 			      << (grub_le_to_cpu16 (data->sblock.log2_blksz)
 				  - GRUB_DISK_SECTOR_BITS), 0,
 			      sizeof (tree), (char *) &tree))
 	    return -1;
-	  
+
 	  return getblk (&tree.treehead, &tree.extents[0]);
 	}
-      
+
       return -1;
     }
-  
+
   return getblk (&inode->file.tree, &inode->file.extents[0]);
 }
 
@@ -312,16 +309,16 @@ grub_jfs_read_inode (struct grub_jfs_data *data, int ino,
   if (grub_disk_read (data->disk,
 		      iagblk << (grub_le_to_cpu16 (data->sblock.log2_blksz)
 				 - GRUB_DISK_SECTOR_BITS), 0,
-		      sizeof (struct grub_jfs_iag), (char *) &iag))
+		      sizeof (struct grub_jfs_iag), &iag))
     return grub_errno;
-  
+
   inoblk = grub_le_to_cpu32 (iag.inodes[inoext].blk2);
   inoblk <<= (grub_le_to_cpu16 (data->sblock.log2_blksz)
 	      - GRUB_DISK_SECTOR_BITS);
   inoblk += inonum;
-  
+
   if (grub_disk_read (data->disk, inoblk, 0,
-		      sizeof (struct grub_jfs_inode), (char *) inode))
+		      sizeof (struct grub_jfs_inode), inode))
     return grub_errno;
 
   return 0;
@@ -339,32 +336,32 @@ grub_jfs_mount (grub_disk_t disk)
 
   /* Read the superblock.  */
   if (grub_disk_read (disk, GRUB_JFS_SBLOCK, 0,
-		      sizeof (struct grub_jfs_sblock), (char *) &data->sblock))
+		      sizeof (struct grub_jfs_sblock), &data->sblock))
     goto fail;
-  
+
   if (grub_strncmp ((char *) (data->sblock.magic), "JFS1", 4))
     {
       grub_error (GRUB_ERR_BAD_FS, "not a jfs filesystem");
       goto fail;
     }
-  
+
   data->disk = disk;
   data->pos = 0;
   data->linknest = 0;
 
   /* Read the inode of the first fileset.  */
   if (grub_disk_read (data->disk, GRUB_JFS_FS1_INODE_BLK, 0,
-		      sizeof (struct grub_jfs_inode), (char *) &data->fileset))
+		      sizeof (struct grub_jfs_inode), &data->fileset))
     goto fail;
-  
+
   return data;
-  
+
  fail:
   grub_free (data);
-  
+
   if (grub_errno == GRUB_ERR_OUT_OF_RANGE)
     grub_error (GRUB_ERR_BAD_FS, "not a jfs filesystem");
-  
+
   return 0;
 }
 
@@ -375,21 +372,20 @@ grub_jfs_opendir (struct grub_jfs_data *data, struct grub_jfs_inode *inode)
   struct grub_jfs_internal_dirent *de;
   struct grub_jfs_diropen *diro;
   int blk;
-  
+
   de = (struct grub_jfs_internal_dirent *) inode->dir.dirents;
-  
+
   if (!((grub_le_to_cpu32 (inode->mode)
 	 & GRUB_JFS_FILETYPE_MASK) == GRUB_JFS_FILETYPE_DIR))
     {
       grub_error (GRUB_ERR_BAD_FILE_TYPE, "not a directory");
       return 0;
     }
-  
-  diro = grub_malloc (sizeof (struct grub_jfs_diropen));
+
+  diro = grub_zalloc (sizeof (struct grub_jfs_diropen));
   if (!diro)
     return 0;
-  
-  diro->index = 0;
+
   diro->data = data;
   diro->inode = inode;
 
@@ -400,7 +396,6 @@ grub_jfs_opendir (struct grub_jfs_data *data, struct grub_jfs_inode *inode)
       diro->next_leaf = (struct grub_jfs_leaf_next_dirent *) de;
       diro->sorted = (char *) (inode->dir.header.sorted);
       diro->count = inode->dir.header.count;
-      diro->dirpage = 0;
 
       return diro;
     }
@@ -411,10 +406,10 @@ grub_jfs_opendir (struct grub_jfs_data *data, struct grub_jfs_inode *inode)
       grub_free (diro);
       return 0;
     }
-  
+
   blk = grub_le_to_cpu32 (de[inode->dir.header.sorted[0]].ex.blk2);
   blk <<= (grub_le_to_cpu16 (data->sblock.log2_blksz) - GRUB_DISK_SECTOR_BITS);
-  
+
   /* Read in the nodes until we are on the leaf node level.  */
   do
     {
@@ -439,7 +434,7 @@ grub_jfs_opendir (struct grub_jfs_data *data, struct grub_jfs_inode *inode)
   diro->next_leaf = diro->dirpage->next_dirent;
   diro->sorted = &diro->dirpage->sorted[diro->dirpage->header.sindex * 32];
   diro->count = diro->dirpage->header.count;
-  
+
   return diro;
 }
 
@@ -464,31 +459,31 @@ grub_jfs_getent (struct grub_jfs_diropen *diro)
   int len;
   int nextent;
   grub_uint16_t filename[255];
-  
+
   auto void addstr (grub_uint16_t *uname, int ulen);
-  
+
   /* Add the unicode string to the utf16 filename buffer.  */
   void addstr (grub_uint16_t *name, int ulen)
     {
       while (ulen--)
 	filename[strpos++] = *(name++);
     }
-  
+
   /* The last node, read in more.  */
   if (diro->index == diro->count)
     {
       unsigned int next;
-      
-      /* If the inode contains the entrie tree or if this was the last
+
+      /* If the inode contains the entry tree or if this was the last
 	 node, there is nothing to read.  */
       if ((diro->inode->file.tree.flags & GRUB_JFS_TREE_LEAF)
 	  || !grub_le_to_cpu64 (diro->dirpage->header.nextb))
 	return GRUB_ERR_OUT_OF_RANGE;
-      
+
       next = grub_le_to_cpu64 (diro->dirpage->header.nextb);
       next <<= (grub_le_to_cpu16 (diro->data->sblock.log2_blksz)
 		- GRUB_DISK_SECTOR_BITS);
-      
+
       if (grub_disk_read (diro->data->disk, next, 0,
 			  grub_le_to_cpu32 (diro->data->sblock.blksz),
 			  diro->dirpage->sorted))
@@ -503,18 +498,18 @@ grub_jfs_getent (struct grub_jfs_diropen *diro)
 
   leaf = &diro->leaf[(int) diro->sorted[diro->index]];
   next_leaf = &diro->next_leaf[diro->index];
-  
+
   len = leaf->len;
   if (!len)
     {
       diro->index++;
       return grub_jfs_getent (diro);
     }
-  
+
   addstr (leaf->namepart, len < 11 ? len : 11);
   diro->ino = grub_le_to_cpu32 (leaf->inode);
   len -= 11;
-  
+
   /* Move down to the leaf level.  */
   nextent = leaf->next;
   if (leaf->next != 255)
@@ -522,7 +517,7 @@ grub_jfs_getent (struct grub_jfs_diropen *diro)
       {
  	next_leaf = &diro->next_leaf[nextent];
 	addstr (next_leaf->namepart, len < 15 ? len : 15 );
-	
+
 	len -= 15;
 	nextent = next_leaf->next;
       } while (next_leaf->next != 255 && len > 0);
@@ -531,7 +526,7 @@ grub_jfs_getent (struct grub_jfs_diropen *diro)
 
   /* Convert the temporary UTF16 filename to UTF8.  */
   *grub_utf16_to_utf8 ((grub_uint8_t *) (diro->name), filename, strpos) = '\0';
-  
+
   return 0;
 }
 
@@ -540,7 +535,7 @@ grub_jfs_getent (struct grub_jfs_diropen *diro)
    POS.  Return the amount of read bytes in READ.  */
 static grub_ssize_t
 grub_jfs_read_file (struct grub_jfs_data *data,
-		    void (*read_hook) (grub_disk_addr_t sector,
+		    void NESTED_FUNC_ATTR (*read_hook) (grub_disk_addr_t sector,
 				       unsigned offset, unsigned length),
 		    int pos, grub_size_t len, char *buf)
 {
@@ -553,15 +548,15 @@ grub_jfs_read_file (struct grub_jfs_data *data,
 
   blockcnt = ((len + pos + grub_le_to_cpu32 (data->sblock.blksz) - 1)
 	      / grub_le_to_cpu32 (data->sblock.blksz));
-  
+
   for (i = pos / grub_le_to_cpu32 (data->sblock.blksz); i < blockcnt; i++)
     {
       int blknr;
       int blockoff = pos % grub_le_to_cpu32 (data->sblock.blksz);
       int blockend = grub_le_to_cpu32 (data->sblock.blksz);
-      
+
       int skipfirst = 0;
-      
+
       blknr = grub_jfs_blkno (data, &data->currinode, i);
       if (grub_errno)
 	return -1;
@@ -570,31 +565,31 @@ grub_jfs_read_file (struct grub_jfs_data *data,
       if (i == blockcnt - 1)
 	{
 	  blockend = (len + pos) % grub_le_to_cpu32 (data->sblock.blksz);
-	  
+
 	  if (!blockend)
 	    blockend = grub_le_to_cpu32 (data->sblock.blksz);
 	}
-      
+
       /* First block.  */
       if (i == (pos / (int) grub_le_to_cpu32 (data->sblock.blksz)))
 	{
 	  skipfirst = blockoff;
 	  blockend -= skipfirst;
 	}
-      
+
       data->disk->read_hook = read_hook;
       grub_disk_read (data->disk,
 		      blknr << (grub_le_to_cpu16 (data->sblock.log2_blksz)
 				- GRUB_DISK_SECTOR_BITS),
 		      skipfirst, blockend, buf);
-      
+
       data->disk->read_hook = 0;
       if (grub_errno)
 	return -1;
-      
+
       buf += grub_le_to_cpu32 (data->sblock.blksz) - skipfirst;
     }
-  
+
   return len;
 }
 
@@ -609,14 +604,14 @@ grub_jfs_find_file (struct grub_jfs_data *data, const char *path)
   char *next;
   unsigned int pos = 0;
   struct grub_jfs_diropen *diro;
-  
+
   grub_strncpy (fpath, path, grub_strlen (path) + 1);
-  
+
   if (grub_jfs_read_inode (data, GRUB_JFS_AGGR_INODE, &data->currinode))
     return grub_errno;
 
-  /* Skip the first slash.  */
-  if (name[0] == '/')
+  /* Skip the first slashes.  */
+  while (*name == '/')
     {
       name++;
       if (!*name)
@@ -627,35 +622,37 @@ grub_jfs_find_file (struct grub_jfs_data *data, const char *path)
   next = grub_strchr (name, '/');
   if (next)
     {
-      next[0] = '\0';
-      next++;
+      while (*next == '/')
+	{
+	  next[0] = '\0';
+	  next++;
+	}
     }
-  
   diro = grub_jfs_opendir (data, &data->currinode);
   if (!diro)
     return grub_errno;
-    
+
   for (;;)
     {
       if (grub_strlen (name) == 0)
 	return GRUB_ERR_NONE;
-      
+
       if (grub_jfs_getent (diro) == GRUB_ERR_OUT_OF_RANGE)
 	break;
-      
+
       /* Check if the current direntry matches the current part of the
 	 pathname.  */
       if (!grub_strcmp (name, diro->name))
 	{
 	  int ino = diro->ino;
 	  int dirino = grub_le_to_cpu32 (data->currinode.inode);
-	  
+
 	  grub_jfs_closedir (diro);
 	  diro = 0;
-	  
+
 	  if (grub_jfs_read_inode (data, ino, &data->currinode))
 	    break;
-	  
+
 	  /* Check if this is a symlink.  */
 	  if ((grub_le_to_cpu32 (data->currinode.mode)
 	       & GRUB_JFS_FILETYPE_MASK) == GRUB_JFS_FILETYPE_LNK)
@@ -664,12 +661,12 @@ grub_jfs_find_file (struct grub_jfs_data *data, const char *path)
 	      if (grub_errno)
 		return grub_errno;
 	    }
-	  
+
 	  if (!next)
 	    return 0;
 
 	  pos = 0;
-	  
+
 	  name = next;
 	  next = grub_strchr (name, '/');
 	  if (next)
@@ -677,12 +674,12 @@ grub_jfs_find_file (struct grub_jfs_data *data, const char *path)
 	      next[0] = '\0';
 	      next++;
 	    }
-	  
+
 	  /* Open this directory for reading dirents.  */
 	  diro = grub_jfs_opendir (data, &data->currinode);
 	  if (!diro)
 	    return grub_errno;
-	  
+
 	  continue;
 	}
     }
@@ -701,40 +698,39 @@ grub_jfs_lookup_symlink (struct grub_jfs_data *data, int ino)
 
   if (++data->linknest > GRUB_JFS_MAX_SYMLNK_CNT)
     return grub_error (GRUB_ERR_SYMLINK_LOOP, "too deep nesting of symlinks");
-  
+
   if (size <= 128)
     grub_strncpy (symlink, (char *) (data->currinode.symlink.path), 128);
   else if (grub_jfs_read_file (data, 0, 0, size, symlink) < 0)
     return grub_errno;
 
   symlink[size] = '\0';
-  
+
   /* The symlink is an absolute path, go back to the root inode.  */
   if (symlink[0] == '/')
     ino = 2;
-  
+
   /* Now load in the old inode.  */
   if (grub_jfs_read_inode (data, ino, &data->currinode))
     return grub_errno;
-  
+
   grub_jfs_find_file (data, symlink);
   if (grub_errno)
     grub_error (grub_errno, "Can not follow symlink `%s'.", symlink);
-  
+
   return grub_errno;
 }
 
 
 static grub_err_t
-grub_jfs_dir (grub_device_t device, const char *path, 
-	      int (*hook) (const char *filename, int dir))
+grub_jfs_dir (grub_device_t device, const char *path,
+	      int (*hook) (const char *filename,
+			   const struct grub_dirhook_info *info))
 {
   struct grub_jfs_data *data = 0;
   struct grub_jfs_diropen *diro = 0;
 
-#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
-#endif
 
   data = grub_jfs_mount (device->disk);
   if (!data)
@@ -742,7 +738,7 @@ grub_jfs_dir (grub_device_t device, const char *path,
 
   if (grub_jfs_find_file (data, path))
     goto fail;
-  
+
   diro = grub_jfs_opendir (data, &data->currinode);
   if (!diro)
     goto fail;
@@ -751,17 +747,18 @@ grub_jfs_dir (grub_device_t device, const char *path,
   while (grub_jfs_getent (diro) != GRUB_ERR_OUT_OF_RANGE)
     {
       struct grub_jfs_inode inode;
-      int isdir;
-      
+      struct grub_dirhook_info info;
+      grub_memset (&info, 0, sizeof (info));
+
       if (grub_jfs_read_inode (data, diro->ino, &inode))
 	goto fail;
-      
-      isdir = (grub_le_to_cpu32 (inode.mode)
-	       & GRUB_JFS_FILETYPE_MASK) == GRUB_JFS_FILETYPE_DIR;
-      if (hook (diro->name, isdir))
+
+      info.dir = (grub_le_to_cpu32 (inode.mode)
+		  & GRUB_JFS_FILETYPE_MASK) == GRUB_JFS_FILETYPE_DIR;
+      if (hook (diro->name, &info))
 	goto fail;
     }
-  
+
   /* XXX: GRUB_ERR_OUT_OF_RANGE is used for the last dirent.  */
   if (grub_errno == GRUB_ERR_OUT_OF_RANGE)
     grub_errno = 0;
@@ -770,9 +767,7 @@ grub_jfs_dir (grub_device_t device, const char *path,
   grub_jfs_closedir (diro);
   grub_free (data);
 
-#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
-#endif
 
   return grub_errno;
 }
@@ -784,18 +779,16 @@ grub_jfs_open (struct grub_file *file, const char *name)
 {
   struct grub_jfs_data *data;
 
-#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
-#endif
 
   data = grub_jfs_mount (file->device->disk);
   if (!data)
     goto fail;
-  
+
   grub_jfs_find_file (data, name);
   if (grub_errno)
     goto fail;
-  
+
   /* It is only possible for open regular files.  */
   if (! ((grub_le_to_cpu32 (data->currinode.mode)
 	  & GRUB_JFS_FILETYPE_MASK) == GRUB_JFS_FILETYPE_REG))
@@ -803,30 +796,28 @@ grub_jfs_open (struct grub_file *file, const char *name)
       grub_error (GRUB_ERR_BAD_FILE_TYPE, "not a regular file");
       goto fail;
     }
-  
+
   file->data = data;
   file->size = grub_le_to_cpu64 (data->currinode.size);
-  
+
   return 0;
-  
+
  fail:
 
-#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
-#endif
-  
+
   grub_free (data);
-  
-  return grub_errno;;
+
+  return grub_errno;
 }
 
 
 static grub_ssize_t
 grub_jfs_read (grub_file_t file, char *buf, grub_size_t len)
 {
-  struct grub_jfs_data *data = 
+  struct grub_jfs_data *data =
     (struct grub_jfs_data *) file->data;
-  
+
   return grub_jfs_read_file (data, file->read_hook, file->offset, len, buf);
 }
 
@@ -835,11 +826,9 @@ static grub_err_t
 grub_jfs_close (grub_file_t file)
 {
   grub_free (file->data);
-  
-#ifndef GRUB_UTIL
+
   grub_dl_unref (my_mod);
-#endif
-  
+
   return GRUB_ERR_NONE;
 }
 
@@ -849,12 +838,12 @@ grub_jfs_label (grub_device_t device, char **label)
 {
   struct grub_jfs_data *data;
   data = grub_jfs_mount (device->disk);
-  
+
   if (data)
     *label = grub_strndup ((char *) (data->sblock.volname), 11);
   else
     *label = 0;
-  
+
   return grub_errno;
 }
 
@@ -873,9 +862,7 @@ static struct grub_fs grub_jfs_fs =
 GRUB_MOD_INIT(jfs)
 {
   grub_fs_register (&grub_jfs_fs);
-#ifndef GRUB_UTIL
   my_mod = mod;
-#endif
 }
 
 GRUB_MOD_FINI(jfs)
