@@ -1,7 +1,7 @@
 /* udf.c - Universal Disk Format filesystem.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2008,2009  Free Software Foundation, Inc.
+ *  Copyright (C) 2008  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -356,7 +356,9 @@ struct grub_fshelp_node
   int part_ref;
 };
 
+#ifndef GRUB_UTIL
 static grub_dl_t my_mod;
+#endif
 
 static grub_uint32_t
 grub_udf_get_block (struct grub_udf_data *data,
@@ -390,7 +392,7 @@ grub_udf_read_icb (struct grub_udf_data *data,
 
   if (grub_disk_read (data->disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
 		      sizeof (struct grub_udf_file_entry),
-		      &node->fe))
+		      (char *) &node->fe))
     return grub_errno;
 
   if ((U16 (node->fe.tag.tag_ident) != GRUB_UDF_TAG_IDENT_FE) &&
@@ -402,12 +404,11 @@ grub_udf_read_icb (struct grub_udf_data *data,
   return 0;
 }
 
-static grub_disk_addr_t
-grub_udf_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
+static int
+grub_udf_read_block (grub_fshelp_node_t node, int fileblock)
 {
   char *ptr;
   int len;
-  grub_disk_addr_t filebytes;
 
   if (U16 (node->fe.tag.tag_ident) == GRUB_UDF_TAG_IDENT_FE)
     {
@@ -426,17 +427,16 @@ grub_udf_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
       struct grub_udf_short_ad *ad = (struct grub_udf_short_ad *) ptr;
 
       len /= sizeof (struct grub_udf_short_ad);
-      filebytes = fileblock * GRUB_UDF_BLKSZ;
       while (len > 0)
 	{
-	  if (filebytes < U32 (ad->length))
+	  if (fileblock < (int) U32 (ad->length))
 	    return ((U32 (ad->position) & GRUB_UDF_EXT_MASK) ? 0 :
                     (grub_udf_get_block (node->data,
                                          node->part_ref,
                                          ad->position)
-                     + (filebytes / GRUB_UDF_BLKSZ)));
+                     + fileblock));
 
-	  filebytes -= U32 (ad->length);
+	  fileblock -= U32 (ad->length);
 	  ad++;
 	  len--;
 	}
@@ -446,17 +446,16 @@ grub_udf_read_block (grub_fshelp_node_t node, grub_disk_addr_t fileblock)
       struct grub_udf_long_ad *ad = (struct grub_udf_long_ad *) ptr;
 
       len /= sizeof (struct grub_udf_long_ad);
-      filebytes = fileblock * GRUB_UDF_BLKSZ;
       while (len > 0)
 	{
-	  if (filebytes < U32 (ad->length))
+	  if (fileblock < (int) U32 (ad->length))
 	    return ((U32 (ad->block.block_num) & GRUB_UDF_EXT_MASK) ?  0 :
                     (grub_udf_get_block (node->data,
                                          ad->block.part_ref,
                                          ad->block.block_num)
-		     + (filebytes / GRUB_UDF_BLKSZ)));
+                    + fileblock));
 
-	  filebytes -= U32 (ad->length);
+	  fileblock -= U32 (ad->length);
 	  ad++;
 	  len--;
 	}
@@ -523,9 +522,9 @@ grub_udf_mount (grub_disk_t disk)
       struct grub_udf_vrs vrs;
 
       if (grub_disk_read (disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
-			  sizeof (struct grub_udf_vrs), &vrs))
+			  sizeof (struct grub_udf_vrs), (char *) &vrs))
 	{
-	  grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	  grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	  goto fail;
 	}
 
@@ -539,7 +538,7 @@ grub_udf_mount (grub_disk_t disk)
 	  (grub_memcmp (vrs.magic, GRUB_UDF_STD_IDENT_CDW02, 5)) &&
 	  (grub_memcmp (vrs.magic, GRUB_UDF_STD_IDENT_TEA01, 5)))
 	{
-	  grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	  grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	  goto fail;
 	}
     }
@@ -550,9 +549,9 @@ grub_udf_mount (grub_disk_t disk)
       struct grub_udf_avdp avdp;
 
       if (grub_disk_read (disk, *sblklist << GRUB_UDF_LOG2_BLKSZ, 0,
-			  sizeof (struct grub_udf_avdp), &avdp))
+			  sizeof (struct grub_udf_avdp), (char *) &avdp))
 	{
-	  grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	  grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	  goto fail;
 	}
 
@@ -565,21 +564,21 @@ grub_udf_mount (grub_disk_t disk)
       sblklist++;
       if (*sblklist == 0)
 	{
-	  grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	  grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	  goto fail;
 	}
     }
 
   data->npd = data->npm = 0;
-  /* Locate Partition Descriptor (PD) and Logical Volume Descriptor (LVD).  */
+  /* Locate Partiton Descriptor (PD) and Logical Volume Descriptor (LVD).  */
   while (1)
     {
       struct grub_udf_tag tag;
 
       if (grub_disk_read (disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
-			  sizeof (struct grub_udf_tag), &tag))
+			  sizeof (struct grub_udf_tag), (char *) &tag))
 	{
-	  grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	  grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	  goto fail;
 	}
 
@@ -594,9 +593,9 @@ grub_udf_mount (grub_disk_t disk)
 
 	  if (grub_disk_read (disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
 			      sizeof (struct grub_udf_pd),
-			      &data->pds[data->npd]))
+			      (char *) &data->pds[data->npd]))
 	    {
-	      grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	      grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	      goto fail;
 	    }
 
@@ -610,9 +609,9 @@ grub_udf_mount (grub_disk_t disk)
 
 	  if (grub_disk_read (disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
 			      sizeof (struct grub_udf_lvd),
-			      &data->lvd))
+			      (char *) &data->lvd))
 	    {
-	      grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+	      grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
 	      goto fail;
 	    }
 
@@ -673,9 +672,9 @@ grub_udf_mount (grub_disk_t disk)
     goto fail;
 
   if (grub_disk_read (disk, block << GRUB_UDF_LOG2_BLKSZ, 0,
-		      sizeof (struct grub_udf_fileset), &root_fs))
+		      sizeof (struct grub_udf_fileset), (char *) &root_fs))
     {
-      grub_error (GRUB_ERR_BAD_FS, "not an UDF filesystem");
+      grub_error (GRUB_ERR_BAD_FS, "not an udf filesystem");
       goto fail;
     }
 
@@ -769,8 +768,7 @@ grub_udf_iterate_dir (grub_fshelp_node_t dir,
 
 static grub_err_t
 grub_udf_dir (grub_device_t device, const char *path,
-	      int (*hook) (const char *filename,
-			   const struct grub_dirhook_info *info))
+	      int (*hook) (const char *filename, int dir))
 {
   struct grub_udf_data *data = 0;
   struct grub_fshelp_node rootnode;
@@ -784,14 +782,19 @@ grub_udf_dir (grub_device_t device, const char *path,
 				enum grub_fshelp_filetype filetype,
 				grub_fshelp_node_t node)
   {
-      struct grub_dirhook_info info;
-      grub_memset (&info, 0, sizeof (info));
-      info.dir = ((filetype & GRUB_FSHELP_TYPE_MASK) == GRUB_FSHELP_DIR);
-      grub_free (node);
-      return hook (filename, &info);
+    grub_free (node);
+
+    if (filetype == GRUB_FSHELP_DIR)
+      return hook (filename, 1);
+    else
+      return hook (filename, 0);
+
+    return 0;
   }
 
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
+#endif
 
   data = grub_udf_mount (device->disk);
   if (!data)
@@ -813,7 +816,9 @@ grub_udf_dir (grub_device_t device, const char *path,
 fail:
   grub_free (data);
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return grub_errno;
 }
@@ -825,7 +830,9 @@ grub_udf_open (struct grub_file *file, const char *name)
   struct grub_fshelp_node rootnode;
   struct grub_fshelp_node *foundnode;
 
+#ifndef GRUB_UTIL
   grub_dl_ref (my_mod);
+#endif
 
   data = grub_udf_mount (file->device->disk);
   if (!data)
@@ -846,7 +853,9 @@ grub_udf_open (struct grub_file *file, const char *name)
   return 0;
 
 fail:
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   grub_free (data);
 
@@ -872,7 +881,9 @@ grub_udf_close (grub_file_t file)
       grub_free (node);
     }
 
+#ifndef GRUB_UTIL
   grub_dl_unref (my_mod);
+#endif
 
   return GRUB_ERR_NONE;
 }
@@ -907,7 +918,9 @@ static struct grub_fs grub_udf_fs = {
 GRUB_MOD_INIT (udf)
 {
   grub_fs_register (&grub_udf_fs);
+#ifndef GRUB_UTIL
   my_mod = mod;
+#endif
 }
 
 GRUB_MOD_FINI (udf)
