@@ -1,7 +1,7 @@
 /* efi.h - declare EFI types and functions */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2006,2007,2008,2009  Free Software Foundation, Inc.
+ *  Copyright (C) 2006,2007  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #define GRUB_EFI_API_HEADER	1
 
 #include <grub/types.h>
-#include <grub/symbol.h>
 
 /* For consistency and safety, we name the EFI-defined types differently.
    All names are transformed into lower case, _t appended, and
@@ -41,15 +40,15 @@
 #define GRUB_EFI_TPL_NOTIFY		16
 #define GRUB_EFI_TPL_HIGH_LEVEL		31
 
-#define GRUB_EFI_MEMORY_UC	0x0000000000000001LL
-#define GRUB_EFI_MEMORY_WC	0x0000000000000002LL
-#define GRUB_EFI_MEMORY_WT	0x0000000000000004LL
-#define GRUB_EFI_MEMORY_WB	0x0000000000000008LL
-#define GRUB_EFI_MEMORY_UCE	0x0000000000000010LL
-#define GRUB_EFI_MEMORY_WP	0x0000000000001000LL
-#define GRUB_EFI_MEMORY_RP	0x0000000000002000LL
-#define GRUB_EFI_MEMORY_XP	0x0000000000004000LL
-#define GRUB_EFI_MEMORY_RUNTIME	0x8000000000000000LL
+#define GRUB_EFI_MEMORY_UC	0x0000000000000001
+#define GRUB_EFI_MEMORY_WC	0x0000000000000002
+#define GRUB_EFI_MEMORY_WT	0x0000000000000004
+#define GRUB_EFI_MEMORY_WB	0x0000000000000008
+#define GRUB_EFI_MEMORY_UCE	0x0000000000000010
+#define GRUB_EFI_MEMORY_WP	0x0000000000001000
+#define GRUB_EFI_MEMORY_RP	0x0000000000002000
+#define GRUB_EFI_MEMORY_XP	0x0000000000004000
+#define GRUB_EFI_MEMORY_RUNTIME	0x8000000000000000
 
 #define GRUB_EFI_OPEN_PROTOCOL_BY_HANDLE_PROTOCOL	0x00000001
 #define GRUB_EFI_OPEN_PROTOCOL_GET_PROTOCOL		0x00000002
@@ -84,167 +83,11 @@
     { 0x8e, 0x39, 0x00, 0xa0, 0xc9, 0x69, 0x72, 0x3b } \
   }
 
-#define GRUB_EFI_SERIAL_IO_GUID \
-  { 0xbb25cf6f, 0xf1d4, 0x11d2, \
-    { 0x9a, 0x0c, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0xfd } \
-  }
-
-#define GRUB_EFI_SIMPLE_NETWORK_GUID	\
-  { 0xa19832b9, 0xac25, 0x11d3, \
-    { 0x9a, 0x2d, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
-#define GRUB_EFI_PXE_GUID	\
-  { 0x03c4e603, 0xac28, 0x11d3, \
-    { 0x9a, 0x2d, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
 #define GRUB_EFI_DEVICE_PATH_GUID	\
   { 0x09576e91, 0x6d3f, 0x11d2, \
     { 0x8e, 0x39, 0x00, 0xa0, 0xc9, 0x69, 0x72, 0x3b } \
   }
 
-#define GRUB_EFI_MPS_TABLE_GUID	\
-  { 0xeb9d2d2f, 0x2d88, 0x11d3, \
-    { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
-#define GRUB_EFI_ACPI_TABLE_GUID	\
-  { 0xeb9d2d30, 0x2d88, 0x11d3, \
-    { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
-#define GRUB_EFI_ACPI_20_TABLE_GUID	\
-  { 0x8868e871, 0xe4f1, 0x11d3, \
-    { 0xbc, 0x22, 0x0, 0x80, 0xc7, 0x3c, 0x88, 0x81 } \
-  }
-
-#define GRUB_EFI_SMBIOS_TABLE_GUID	\
-  { 0xeb9d2d31, 0x2d88, 0x11d3, \
-    { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
-#define GRUB_EFI_SAL_TABLE_GUID \
-  { 0xeb9d2d32, 0x2d88, 0x11d3, \
-      { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
-  }
-
-#define GRUB_EFI_HCDP_TABLE_GUID \
-  { 0xf951938d, 0x620b, 0x42ef, \
-      { 0x82, 0x79, 0xa8, 0x4b, 0x79, 0x61, 0x78, 0x98 } \
-  }
-
-struct grub_efi_sal_system_table
-{
-  grub_uint32_t signature;
-  grub_uint32_t total_table_len;
-  grub_uint16_t sal_rev;
-  grub_uint16_t entry_count;
-  grub_uint8_t checksum;
-  grub_uint8_t reserved1[7];
-  grub_uint16_t sal_a_version;
-  grub_uint16_t sal_b_version;
-  grub_uint8_t oem_id[32];
-  grub_uint8_t product_id[32];
-  grub_uint8_t reserved2[8];
-  grub_uint8_t entries[0];
-};
-
-enum
-  {
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_ENTRYPOINT_DESCRIPTOR = 0,
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_MEMORY_DESCRIPTOR = 1,
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_PLATFORM_FEATURES = 2,
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_TRANSLATION_REGISTER_DESCRIPTOR = 3,
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_PURGE_TRANSLATION_COHERENCE = 4,
-    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_AP_WAKEUP = 5
-  };
-
-struct grub_efi_sal_system_table_entrypoint_descriptor
-{
-  grub_uint8_t type;
-  grub_uint8_t pad[7];
-  grub_uint64_t pal_proc_addr;
-  grub_uint64_t sal_proc_addr;
-  grub_uint64_t global_data_ptr;
-  grub_uint64_t reserved[2];
-};
-
-struct grub_efi_sal_system_table_memory_descriptor
-{
-  grub_uint8_t type;
-  grub_uint8_t sal_used;
-  grub_uint8_t attr;
-  grub_uint8_t ar;
-  grub_uint8_t attr_mask;
-  grub_uint8_t mem_type;
-  grub_uint8_t usage;
-  grub_uint8_t unknown;
-  grub_uint64_t addr;
-  grub_uint64_t len;
-  grub_uint64_t unknown2;
-};
-
-struct grub_efi_sal_system_table_platform_features
-{
-  grub_uint8_t type;
-  grub_uint8_t flags;
-  grub_uint8_t reserved[14];
-};
-
-struct grub_efi_sal_system_table_translation_register_descriptor
-{
-  grub_uint8_t type;
-  grub_uint8_t register_type;
-  grub_uint8_t register_number;
-  grub_uint8_t reserved[5];
-  grub_uint64_t addr;
-  grub_uint64_t page_size;
-  grub_uint64_t reserver;
-};
-
-struct grub_efi_sal_system_table_purge_translation_coherence
-{
-  grub_uint8_t type;
-  grub_uint8_t reserved[3];  
-  grub_uint32_t ndomains;
-  grub_uint64_t coherence;
-};
-
-struct grub_efi_sal_system_table_ap_wakeup
-{
-  grub_uint8_t type;
-  grub_uint8_t mechanism;
-  grub_uint8_t reserved[6];
-  grub_uint64_t vector;
-};
-
-enum
-  {
-    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_BUSLOCK = 1,
-    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_IRQREDIRECT = 2,
-    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_IPIREDIRECT = 4,
-    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_ITCDRIFT = 8,
-  };
-
-typedef enum grub_efi_parity_type
-  {
-    GRUB_EFI_SERIAL_DEFAULT_PARITY,
-    GRUB_EFI_SERIAL_NO_PARITY,
-    GRUB_EFI_SERIAL_EVEN_PARITY,
-    GRUB_EFI_SERIAL_ODD_PARITY
-  }
-grub_efi_parity_type_t;
-
-typedef enum grub_efi_stop_bits
-  {
-    GRUB_EFI_SERIAL_DEFAULT_STOP_BITS,
-    GRUB_EFI_SERIAL_1_STOP_BIT,
-    GRUB_EFI_SERIAL_1_5_STOP_BITS,
-    GRUB_EFI_SERIAL_2_STOP_BITS
-  }
-  grub_efi_stop_bits_t;
-
 /* Enumerations.  */
 enum grub_efi_timer_delay
   {
@@ -320,8 +163,6 @@ typedef grub_uint64_t grub_efi_uint64_t;
 typedef grub_uint8_t grub_efi_char8_t;
 typedef grub_uint16_t grub_efi_char16_t;
 
-#define PRIxGRUB_EFI_UINTN_T "lx"
-
 typedef grub_efi_intn_t grub_efi_status_t;
 
 #define GRUB_EFI_ERROR_CODE(value)	\
@@ -394,7 +235,7 @@ struct grub_efi_memory_descriptor
   grub_efi_virtual_address_t virtual_start;
   grub_efi_uint64_t num_pages;
   grub_efi_uint64_t attribute;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_memory_descriptor grub_efi_memory_descriptor_t;
 
 /* Device Path definitions.  */
@@ -439,7 +280,7 @@ struct grub_efi_pci_device_path
   grub_efi_device_path_t header;
   grub_efi_uint8_t function;
   grub_efi_uint8_t device;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_pci_device_path grub_efi_pci_device_path_t;
 
 #define GRUB_EFI_PCCARD_DEVICE_PATH_SUBTYPE		2
@@ -448,7 +289,7 @@ struct grub_efi_pccard_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint8_t function;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_pccard_device_path grub_efi_pccard_device_path_t;
 
 #define GRUB_EFI_MEMORY_MAPPED_DEVICE_PATH_SUBTYPE	3
@@ -456,10 +297,10 @@ typedef struct grub_efi_pccard_device_path grub_efi_pccard_device_path_t;
 struct grub_efi_memory_mapped_device_path
 {
   grub_efi_device_path_t header;
-  grub_efi_uint32_t memory_type;
+  grub_efi_memory_type_t memory_type;
   grub_efi_physical_address_t start_address;
   grub_efi_physical_address_t end_address;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_memory_mapped_device_path grub_efi_memory_mapped_device_path_t;
 
 #define GRUB_EFI_VENDOR_DEVICE_PATH_SUBTYPE		4
@@ -469,7 +310,7 @@ struct grub_efi_vendor_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_vendor_device_path grub_efi_vendor_device_path_t;
 
 #define GRUB_EFI_CONTROLLER_DEVICE_PATH_SUBTYPE		5
@@ -478,7 +319,7 @@ struct grub_efi_controller_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint32_t controller_number;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_controller_device_path grub_efi_controller_device_path_t;
 
 /* ACPI Device Path.  */
@@ -491,7 +332,7 @@ struct grub_efi_acpi_device_path
   grub_efi_device_path_t header;
   grub_efi_uint32_t hid;
   grub_efi_uint32_t uid;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_acpi_device_path grub_efi_acpi_device_path_t;
 
 #define GRUB_EFI_EXPANDED_ACPI_DEVICE_PATH_SUBTYPE	2
@@ -502,8 +343,8 @@ struct grub_efi_expanded_acpi_device_path
   grub_efi_uint32_t hid;
   grub_efi_uint32_t uid;
   grub_efi_uint32_t cid;
-  char hidstr[0];
-} __attribute__ ((packed));
+  char hidstr[1];
+};
 typedef struct grub_efi_expanded_acpi_device_path grub_efi_expanded_acpi_device_path_t;
 
 #define GRUB_EFI_EXPANDED_ACPI_HIDSTR(dp)	\
@@ -526,7 +367,7 @@ struct grub_efi_atapi_device_path
   grub_efi_uint8_t primary_secondary;
   grub_efi_uint8_t slave_master;
   grub_efi_uint16_t lun;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_atapi_device_path grub_efi_atapi_device_path_t;
 
 #define GRUB_EFI_SCSI_DEVICE_PATH_SUBTYPE		2
@@ -536,7 +377,7 @@ struct grub_efi_scsi_device_path
   grub_efi_device_path_t header;
   grub_efi_uint16_t pun;
   grub_efi_uint16_t lun;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_scsi_device_path grub_efi_scsi_device_path_t;
 
 #define GRUB_EFI_FIBRE_CHANNEL_DEVICE_PATH_SUBTYPE	3
@@ -547,7 +388,7 @@ struct grub_efi_fibre_channel_device_path
   grub_efi_uint32_t reserved;
   grub_efi_uint64_t wwn;
   grub_efi_uint64_t lun;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_fibre_channel_device_path grub_efi_fibre_channel_device_path_t;
 
 #define GRUB_EFI_1394_DEVICE_PATH_SUBTYPE		4
@@ -557,7 +398,7 @@ struct grub_efi_1394_device_path
   grub_efi_device_path_t header;
   grub_efi_uint32_t reserved;
   grub_efi_uint64_t guid;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_1394_device_path grub_efi_1394_device_path_t;
 
 #define GRUB_EFI_USB_DEVICE_PATH_SUBTYPE		5
@@ -567,7 +408,7 @@ struct grub_efi_usb_device_path
   grub_efi_device_path_t header;
   grub_efi_uint8_t parent_port_number;
   grub_efi_uint8_t interface;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_usb_device_path grub_efi_usb_device_path_t;
 
 #define GRUB_EFI_USB_CLASS_DEVICE_PATH_SUBTYPE		15
@@ -580,7 +421,7 @@ struct grub_efi_usb_class_device_path
   grub_efi_uint8_t device_class;
   grub_efi_uint8_t device_subclass;
   grub_efi_uint8_t device_protocol;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_usb_class_device_path grub_efi_usb_class_device_path_t;
 
 #define GRUB_EFI_I2O_DEVICE_PATH_SUBTYPE		6
@@ -589,7 +430,7 @@ struct grub_efi_i2o_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint32_t tid;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_i2o_device_path grub_efi_i2o_device_path_t;
 
 #define GRUB_EFI_MAC_ADDRESS_DEVICE_PATH_SUBTYPE	11
@@ -599,7 +440,7 @@ struct grub_efi_mac_address_device_path
   grub_efi_device_path_t header;
   grub_efi_mac_address_t mac_address;
   grub_efi_uint8_t if_type;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_mac_address_device_path grub_efi_mac_address_device_path_t;
 
 #define GRUB_EFI_IPV4_DEVICE_PATH_SUBTYPE		12
@@ -613,7 +454,7 @@ struct grub_efi_ipv4_device_path
   grub_efi_uint16_t remote_port;
   grub_efi_uint16_t protocol;
   grub_efi_uint8_t static_ip_address;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_ipv4_device_path grub_efi_ipv4_device_path_t;
 
 #define GRUB_EFI_IPV6_DEVICE_PATH_SUBTYPE		13
@@ -627,7 +468,7 @@ struct grub_efi_ipv6_device_path
   grub_efi_uint16_t remote_port;
   grub_efi_uint16_t protocol;
   grub_efi_uint8_t static_ip_address;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_ipv6_device_path grub_efi_ipv6_device_path_t;
 
 #define GRUB_EFI_INFINIBAND_DEVICE_PATH_SUBTYPE		9
@@ -640,7 +481,7 @@ struct grub_efi_infiniband_device_path
   grub_efi_uint64_t remote_id;
   grub_efi_uint64_t target_port_id;
   grub_efi_uint64_t device_id;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_infiniband_device_path grub_efi_infiniband_device_path_t;
 
 #define GRUB_EFI_UART_DEVICE_PATH_SUBTYPE		14
@@ -653,7 +494,7 @@ struct grub_efi_uart_device_path
   grub_efi_uint8_t data_bits;
   grub_efi_uint8_t parity;
   grub_efi_uint8_t stop_bits;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_uart_device_path grub_efi_uart_device_path_t;
 
 #define GRUB_EFI_VENDOR_MESSAGING_DEVICE_PATH_SUBTYPE	10
@@ -663,7 +504,7 @@ struct grub_efi_vendor_messaging_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_vendor_messaging_device_path grub_efi_vendor_messaging_device_path_t;
 
 /* Media Device Path.  */
@@ -680,7 +521,7 @@ struct grub_efi_hard_drive_device_path
   grub_efi_uint8_t partition_signature[8];
   grub_efi_uint8_t mbr_type;
   grub_efi_uint8_t signature_type;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_hard_drive_device_path grub_efi_hard_drive_device_path_t;
 
 #define GRUB_EFI_CDROM_DEVICE_PATH_SUBTYPE		2
@@ -691,7 +532,7 @@ struct grub_efi_cdrom_device_path
   grub_efi_uint32_t boot_entry;
   grub_efi_lba_t partition_start;
   grub_efi_lba_t partition_size;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_cdrom_device_path grub_efi_cdrom_device_path_t;
 
 #define GRUB_EFI_VENDOR_MEDIA_DEVICE_PATH_SUBTYPE	3
@@ -701,7 +542,7 @@ struct grub_efi_vendor_media_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_vendor_media_device_path grub_efi_vendor_media_device_path_t;
 
 #define GRUB_EFI_FILE_PATH_DEVICE_PATH_SUBTYPE		4
@@ -710,7 +551,7 @@ struct grub_efi_file_path_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_char16_t path_name[0];
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_file_path_device_path grub_efi_file_path_device_path_t;
 
 #define GRUB_EFI_PROTOCOL_DEVICE_PATH_SUBTYPE		5
@@ -719,19 +560,9 @@ struct grub_efi_protocol_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_guid_t guid;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_protocol_device_path grub_efi_protocol_device_path_t;
 
-#define GRUB_EFI_PIWG_DEVICE_PATH_SUBTYPE		6
-
-struct grub_efi_piwg_device_path
-{
-  grub_efi_device_path_t header;
-  grub_efi_guid_t guid __attribute__ ((packed));
-} __attribute__ ((packed));
-typedef struct grub_efi_piwg_device_path grub_efi_piwg_device_path_t;
-
-
 /* BIOS Boot Specification Device Path.  */
 #define GRUB_EFI_BIOS_DEVICE_PATH_TYPE			5
 
@@ -743,7 +574,7 @@ struct grub_efi_bios_device_path
   grub_efi_uint16_t device_type;
   grub_efi_uint16_t status_flags;
   char description[0];
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_bios_device_path grub_efi_bios_device_path_t;
 
 struct grub_efi_open_protocol_information_entry
@@ -768,7 +599,7 @@ struct grub_efi_time
   grub_efi_int16_t time_zone;
   grub_efi_uint8_t daylight;
   grub_efi_uint8_t pad2;
-} __attribute__ ((packed));
+};
 typedef struct grub_efi_time grub_efi_time_t;
 
 struct grub_efi_time_capabilities
@@ -811,38 +642,38 @@ typedef struct grub_efi_table_header grub_efi_table_header_t;
 struct grub_efi_boot_services
 {
   grub_efi_table_header_t hdr;
-
+  
   grub_efi_tpl_t
   (*raise_tpl) (grub_efi_tpl_t new_tpl);
-
+  
   void
   (*restore_tpl) (grub_efi_tpl_t old_tpl);
-
+  
   grub_efi_status_t
   (*allocate_pages) (grub_efi_allocate_type_t type,
 		     grub_efi_memory_type_t memory_type,
 		     grub_efi_uintn_t pages,
 		     grub_efi_physical_address_t *memory);
-
+  
   grub_efi_status_t
   (*free_pages) (grub_efi_physical_address_t memory,
 		 grub_efi_uintn_t pages);
-
+  
   grub_efi_status_t
   (*get_memory_map) (grub_efi_uintn_t *memory_map_size,
 		     grub_efi_memory_descriptor_t *memory_map,
 		     grub_efi_uintn_t *map_key,
 		     grub_efi_uintn_t *descriptor_size,
 		     grub_efi_uint32_t *descriptor_version);
-
+  
   grub_efi_status_t
   (*allocate_pool) (grub_efi_memory_type_t pool_type,
 		    grub_efi_uintn_t size,
 		    void **buffer);
-
+  
   grub_efi_status_t
   (*free_pool) (void *buffer);
-
+  
   grub_efi_status_t
   (*create_event) (grub_efi_uint32_t type,
 		   grub_efi_tpl_t notify_tpl,
@@ -855,7 +686,7 @@ struct grub_efi_boot_services
   (*set_timer) (grub_efi_event_t event,
 		grub_efi_timer_delay_t type,
 		grub_efi_uint64_t trigger_time);
-
+  
    grub_efi_status_t
    (*wait_for_event) (grub_efi_uintn_t num_events,
 		      grub_efi_event_t *event,
@@ -863,10 +694,10 @@ struct grub_efi_boot_services
 
   grub_efi_status_t
   (*signal_event) (grub_efi_event_t event);
-
+  
   grub_efi_status_t
   (*close_event) (grub_efi_event_t event);
-
+  
   grub_efi_status_t
   (*check_event) (grub_efi_event_t event);
 
@@ -875,13 +706,13 @@ struct grub_efi_boot_services
 				  grub_efi_guid_t *protocol,
 				  grub_efi_interface_type_t interface_type,
 				  void *interface);
-
+  
   grub_efi_status_t
   (*reinstall_protocol_interface) (grub_efi_handle_t handle,
 				   grub_efi_guid_t *protocol,
 				   void *old_interface,
 				   void *new_interface);
-
+  
   grub_efi_status_t
   (*uninstall_protocol_interface) (grub_efi_handle_t handle,
 				   grub_efi_guid_t *protocol,
@@ -891,9 +722,9 @@ struct grub_efi_boot_services
   (*handle_protocol) (grub_efi_handle_t handle,
 		      grub_efi_guid_t *protocol,
 		      void **interface);
-
+  
   void *reserved;
-
+  
   grub_efi_status_t
   (*register_protocol_notify) (grub_efi_guid_t *protocol,
 			       grub_efi_event_t event,
@@ -1023,7 +854,7 @@ struct grub_efi_runtime_services
 {
   grub_efi_table_header_t hdr;
 
-  grub_efi_status_t
+  grub_efi_status_t 
   (*get_time) (grub_efi_time_t *time,
 	       grub_efi_time_capabilities_t *capabilities);
 
@@ -1048,13 +879,9 @@ struct grub_efi_runtime_services
   grub_efi_status_t
   (*convert_pointer) (grub_efi_uintn_t debug_disposition, void **address);
 
-#define GRUB_EFI_GLOBAL_VARIABLE_GUID \
-  { 0x8BE4DF61, 0x93CA, 0x11d2, { 0xAA, 0x0D, 0x00, 0xE0, 0x98, 0x03, 0x2B,0x8C }}
-
-
   grub_efi_status_t
   (*get_variable) (grub_efi_char16_t *variable_name,
-		   const grub_efi_guid_t *vendor_guid,
+		   grub_efi_guid_t *vendor_guid,
 		   grub_efi_uint32_t *attributes,
 		   grub_efi_uintn_t *data_size,
 		   void *data);
@@ -1086,43 +913,19 @@ struct grub_efi_configuration_table
 {
   grub_efi_guid_t vendor_guid;
   void *vendor_table;
-} __attribute__ ((packed));
-typedef struct grub_efi_configuration_table grub_efi_configuration_table_t;
-
-#define GRUB_EFIEMU_SYSTEM_TABLE_SIGNATURE 0x5453595320494249LL
-#define GRUB_EFIEMU_RUNTIME_SERVICES_SIGNATURE 0x56524553544e5552LL
-
-struct grub_efi_serial_io_interface
-{
-  grub_efi_uint32_t revision;
-  void (*reset) (void);
-  grub_efi_status_t (*set_attributes) (struct grub_efi_serial_io_interface *this,
-				       grub_efi_uint64_t speed,
-				       grub_efi_uint32_t fifo_depth,
-				       grub_efi_uint32_t timeout,
-				       grub_efi_parity_type_t parity,
-				       grub_uint8_t word_len,
-				       grub_efi_stop_bits_t stop_bits);
-  void (*set_control_bits) (void);
-  void (*get_control_bits) (void);
-  grub_efi_status_t (*write) (struct grub_efi_serial_io_interface *this,
-			      grub_efi_uintn_t *buf_size,
-			      void *buffer);
-  grub_efi_status_t (*read) (struct grub_efi_serial_io_interface *this,
-			     grub_efi_uintn_t *buf_size,
-			     void *buffer);
 };
+typedef struct grub_efi_configuration_table grub_efi_configuration_table_t;
 
 struct grub_efi_simple_input_interface
 {
   grub_efi_status_t
   (*reset) (struct grub_efi_simple_input_interface *this,
 	    grub_efi_boolean_t extended_verification);
-
+  
   grub_efi_status_t
   (*read_key_stroke) (struct grub_efi_simple_input_interface *this,
 		      grub_efi_input_key_t *key);
-
+  
   grub_efi_event_t wait_for_key;
 };
 typedef struct grub_efi_simple_input_interface grub_efi_simple_input_interface_t;
@@ -1147,7 +950,7 @@ struct grub_efi_simple_text_output_interface
 		 grub_efi_uintn_t *columns,
 		 grub_efi_uintn_t *rows);
 
-  grub_efi_status_t
+  grub_efi_status_t 
   (*set_mode) (struct grub_efi_simple_text_output_interface *this,
 	       grub_efi_uintn_t mode_number);
 
@@ -1166,41 +969,11 @@ struct grub_efi_simple_text_output_interface
   grub_efi_status_t
   (*enable_cursor) (struct grub_efi_simple_text_output_interface *this,
 		    grub_efi_boolean_t visible);
-
+  
   grub_efi_simple_text_output_mode_t *mode;
 };
 typedef struct grub_efi_simple_text_output_interface grub_efi_simple_text_output_interface_t;
 
-typedef grub_uint8_t grub_efi_pxe_packet_t[1472];
-
-typedef struct grub_efi_pxe_mode
-{
-  grub_uint8_t unused[52];
-  grub_efi_pxe_packet_t dhcp_discover;
-  grub_efi_pxe_packet_t dhcp_ack;
-  grub_efi_pxe_packet_t proxy_offer;
-  grub_efi_pxe_packet_t pxe_discover;
-  grub_efi_pxe_packet_t pxe_reply;
-} grub_efi_pxe_mode_t;
-
-typedef struct grub_efi_pxe
-{
-  grub_uint64_t rev;
-  void (*start) (void);
-  void (*stop) (void);
-  void (*dhcp) (void);
-  void (*discover) (void);
-  void (*mftp) (void);
-  void (*udpwrite) (void);
-  void (*udpread) (void);
-  void (*setipfilter) (void);
-  void (*arp) (void);
-  void (*setparams) (void);
-  void (*setstationip) (void);
-  void (*setpackets) (void);
-  struct grub_efi_pxe_mode *mode;
-} grub_efi_pxe_t;
-
 #define GRUB_EFI_BLACK		0x00
 #define GRUB_EFI_BLUE		0x01
 #define GRUB_EFI_GREEN		0x02
@@ -1302,76 +1075,6 @@ struct grub_efi_block_io_media
 };
 typedef struct grub_efi_block_io_media grub_efi_block_io_media_t;
 
-typedef grub_uint8_t grub_efi_mac_t[32];
-
-struct grub_efi_simple_network_mode
-{
-  grub_uint32_t state;
-  grub_uint32_t hwaddr_size;
-  grub_uint32_t media_header_size;
-  grub_uint32_t max_packet_size;
-  grub_uint32_t nvram_size;
-  grub_uint32_t nvram_access_size;
-  grub_uint32_t receive_filter_mask;
-  grub_uint32_t receive_filter_setting;
-  grub_uint32_t max_mcast_filter_count;
-  grub_uint32_t mcast_filter_count;
-  grub_efi_mac_t mcast_filter[16];
-  grub_efi_mac_t current_address;
-  grub_efi_mac_t broadcast_address;
-  grub_efi_mac_t permanent_address;
-  grub_uint8_t if_type;
-  grub_uint8_t mac_changeable;
-  grub_uint8_t multitx_supported;
-  grub_uint8_t media_present_supported;
-  grub_uint8_t media_present;
-};
-
-enum
-  {
-    GRUB_EFI_NETWORK_STOPPED,
-    GRUB_EFI_NETWORK_STARTED,
-    GRUB_EFI_NETWORK_INITIALIZED,
-  };
-
-struct grub_efi_simple_network
-{
-  grub_uint64_t revision;
-  grub_efi_status_t (*start) (struct grub_efi_simple_network *this);
-  void (*stop) (void);
-  grub_efi_status_t (*initialize) (struct grub_efi_simple_network *this,
-				   grub_efi_uintn_t extra_rx,
-				   grub_efi_uintn_t extra_tx);
-  void (*reset) (void);
-  void (*shutdown) (void);
-  void (*receive_filters) (void);
-  void (*station_address) (void);
-  void (*statistics) (void);
-  void (*mcastiptomac) (void);
-  void (*nvdata) (void);
-  grub_efi_status_t (*get_status) (struct grub_efi_simple_network *this,
-				   grub_uint32_t *int_status,
-				   void **txbuf);
-  grub_efi_status_t (*transmit) (struct grub_efi_simple_network *this,
-				 grub_efi_uintn_t header_size,
-				 grub_efi_uintn_t buffer_size,
-				 void *buffer,
-				 grub_efi_mac_t *src_addr,
-				 grub_efi_mac_t *dest_addr,
-				 grub_efi_uint16_t *protocol);
-  grub_efi_status_t (*receive) (struct grub_efi_simple_network *this,
-				grub_efi_uintn_t *header_size,
-				grub_efi_uintn_t *buffer_size,
-				void *buffer,
-				grub_efi_mac_t *src_addr,
-				grub_efi_mac_t *dest_addr,
-				grub_uint16_t *protocol);
-  void (*waitforpacket) (void);
-  struct grub_efi_simple_network_mode *mode;
-};
-typedef struct grub_efi_simple_network grub_efi_simple_network_t;
-
-
 struct grub_efi_block_io
 {
   grub_efi_uint64_t revision;
@@ -1392,75 +1095,4 @@ struct grub_efi_block_io
 };
 typedef struct grub_efi_block_io grub_efi_block_io_t;
 
-#if (GRUB_TARGET_SIZEOF_VOID_P == 4) || defined (__ia64__)
-
-#define efi_call_0(func)		func()
-#define efi_call_1(func, a)		func(a)
-#define efi_call_2(func, a, b)		func(a, b)
-#define efi_call_3(func, a, b, c)	func(a, b, c)
-#define efi_call_4(func, a, b, c, d)	func(a, b, c, d)
-#define efi_call_5(func, a, b, c, d, e)	func(a, b, c, d, e)
-#define efi_call_6(func, a, b, c, d, e, f) func(a, b, c, d, e, f)
-#define efi_call_7(func, a, b, c, d, e, f, g) func(a, b, c, d, e, f, g)
-#define efi_call_10(func, a, b, c, d, e, f, g, h, i, j)	func(a, b, c, d, e, f, g, h, i, j)
-
-#else
-
-#define efi_call_0(func) \
-  efi_wrap_0(func)
-#define efi_call_1(func, a) \
-  efi_wrap_1(func, (grub_uint64_t) (a))
-#define efi_call_2(func, a, b) \
-  efi_wrap_2(func, (grub_uint64_t) (a), (grub_uint64_t) (b))
-#define efi_call_3(func, a, b, c) \
-  efi_wrap_3(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	     (grub_uint64_t) (c))
-#define efi_call_4(func, a, b, c, d) \
-  efi_wrap_4(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	     (grub_uint64_t) (c), (grub_uint64_t) (d))
-#define efi_call_5(func, a, b, c, d, e)	\
-  efi_wrap_5(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e))
-#define efi_call_6(func, a, b, c, d, e, f) \
-  efi_wrap_6(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
-	     (grub_uint64_t) (f))
-#define efi_call_7(func, a, b, c, d, e, f, g) \
-  efi_wrap_7(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
-	     (grub_uint64_t) (f), (grub_uint64_t) (g))
-#define efi_call_10(func, a, b, c, d, e, f, g, h, i, j) \
-  efi_wrap_10(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
-	      (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
-	      (grub_uint64_t) (f), (grub_uint64_t) (g),	(grub_uint64_t) (h), \
-	      (grub_uint64_t) (i), (grub_uint64_t) (j))
-
-grub_uint64_t EXPORT_FUNC(efi_wrap_0) (void *func);
-grub_uint64_t EXPORT_FUNC(efi_wrap_1) (void *func, grub_uint64_t arg1);
-grub_uint64_t EXPORT_FUNC(efi_wrap_2) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2);
-grub_uint64_t EXPORT_FUNC(efi_wrap_3) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2, grub_uint64_t arg3);
-grub_uint64_t EXPORT_FUNC(efi_wrap_4) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2, grub_uint64_t arg3,
-                                       grub_uint64_t arg4);
-grub_uint64_t EXPORT_FUNC(efi_wrap_5) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2, grub_uint64_t arg3,
-                                       grub_uint64_t arg4, grub_uint64_t arg5);
-grub_uint64_t EXPORT_FUNC(efi_wrap_6) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2, grub_uint64_t arg3,
-                                       grub_uint64_t arg4, grub_uint64_t arg5,
-                                       grub_uint64_t arg6);
-grub_uint64_t EXPORT_FUNC(efi_wrap_7) (void *func, grub_uint64_t arg1,
-                                       grub_uint64_t arg2, grub_uint64_t arg3,
-                                       grub_uint64_t arg4, grub_uint64_t arg5,
-                                       grub_uint64_t arg6, grub_uint64_t arg7);
-grub_uint64_t EXPORT_FUNC(efi_wrap_10) (void *func, grub_uint64_t arg1,
-                                        grub_uint64_t arg2, grub_uint64_t arg3,
-                                        grub_uint64_t arg4, grub_uint64_t arg5,
-                                        grub_uint64_t arg6, grub_uint64_t arg7,
-                                        grub_uint64_t arg8, grub_uint64_t arg9,
-                                        grub_uint64_t arg10);
-#endif
-
 #endif /* ! GRUB_EFI_API_HEADER */
