@@ -1,20 +1,19 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1999,2000,2001,2002,2004  Free Software Foundation, Inc.
+ *  Copyright (C) 1999,2000,2001,2002,2004,2007  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GRUB_PC_PARTITION_HEADER
@@ -36,6 +35,7 @@
 #define GRUB_PC_PARTITION_TYPE_FAT16_LT32M	4
 #define GRUB_PC_PARTITION_TYPE_EXTENDED		5
 #define GRUB_PC_PARTITION_TYPE_FAT16_GT32M	6
+#define GRUB_PC_PARTITION_TYPE_NTFS	        7
 #define GRUB_PC_PARTITION_TYPE_FAT32		0xb
 #define GRUB_PC_PARTITION_TYPE_FAT32_LBA	0xc
 #define GRUB_PC_PARTITION_TYPE_FAT16_LBA	0xe
@@ -49,6 +49,7 @@
 #define GRUB_PC_PARTITION_TYPE_FREEBSD		0xa5
 #define GRUB_PC_PARTITION_TYPE_OPENBSD		0xa6
 #define GRUB_PC_PARTITION_TYPE_NETBSD		0xa9
+#define GRUB_PC_PARTITION_TYPE_HFS		0xaf
 #define GRUB_PC_PARTITION_TYPE_GPT_DISK		0xee
 #define GRUB_PC_PARTITION_TYPE_LINUX_RAID	0xfd
 
