@@ -1,25 +1,40 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002,2003,2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2003,2005,2007,2008  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GRUB_TERM_HEADER
 #define GRUB_TERM_HEADER	1
 
+/* Internal codes used by GRUB to represent terminal input.  */
+#define GRUB_TERM_LEFT		2
+#define GRUB_TERM_RIGHT		6
+#define GRUB_TERM_UP		16
+#define GRUB_TERM_DOWN		14
+#define GRUB_TERM_HOME		1
+#define GRUB_TERM_END		5
+#define GRUB_TERM_DC		4
+#define GRUB_TERM_PPAGE		7
+#define GRUB_TERM_NPAGE		3
+#define GRUB_TERM_ESC		'\e'
+#define GRUB_TERM_TAB		'\t'
+#define GRUB_TERM_BACKSPACE	'\b'
+
+#ifndef ASM_FILE
+
 #include <grub/err.h>
 #include <grub/symbol.h>
 #include <grub/types.h>
@@ -165,6 +180,10 @@ struct grub_term
      color is VGA's.  */
   void (*setcolor) (grub_uint8_t normal_color, grub_uint8_t highlight_color);
   
+  /* Get the normal color and the highlight color. The format of each
+     color is VGA's.  */
+  void (*getcolor) (grub_uint8_t *normal_color, grub_uint8_t *highlight_color);
+  
   /* Turn on/off the cursor.  */
   void (*setcursor) (int on);
 
@@ -198,6 +217,8 @@ void EXPORT_FUNC(grub_cls) (void);
 void EXPORT_FUNC(grub_setcolorstate) (grub_term_color_state state);
 void EXPORT_FUNC(grub_setcolor) (grub_uint8_t normal_color,
 				 grub_uint8_t highlight_color);
+void EXPORT_FUNC(grub_getcolor) (grub_uint8_t *normal_color,
+				 grub_uint8_t *highlight_color);
 int EXPORT_FUNC(grub_setcursor) (int on);
 int EXPORT_FUNC(grub_getcursor) (void);
 void EXPORT_FUNC(grub_refresh) (void);
@@ -206,4 +227,6 @@ void EXPORT_FUNC(grub_set_more) (int onoff);
 /* For convenience.  */
 #define GRUB_TERM_ASCII_CHAR(c)	((c) & 0xff)
 
+#endif /* ! ASM_FILE */
+
 #endif /* ! GRUB_TERM_HEADER */
