@@ -1,6 +1,6 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2007, 2008  Free Software Foundation, Inc.
+ *  Copyright (C) 2007  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,22 +19,12 @@
 #ifndef KERNEL_TIME_HEADER
 #define KERNEL_TIME_HEADER	1
 
-#include <grub/types.h>
 #include <grub/symbol.h>
+#include <grub/machine/time.h>
 #include <grub/cpu/time.h>
 
-#if defined (GRUB_MACHINE_EMU) || defined (GRUB_UTIL)
-#define GRUB_TICKS_PER_SECOND 100000
-/* Return the real time in ticks.  */
-grub_uint32_t EXPORT_FUNC (grub_get_rtc) (void);
-#else
-#include <grub/machine/time.h>
-#endif
-
 void EXPORT_FUNC(grub_millisleep) (grub_uint32_t ms);
-grub_uint64_t EXPORT_FUNC(grub_get_time_ms) (void);
-
-grub_uint64_t grub_rtc_get_time_ms (void);
+void EXPORT_FUNC(grub_millisleep_generic) (grub_uint32_t ms);
 
 static __inline void
 grub_sleep (grub_uint32_t s)
@@ -42,6 +32,4 @@ grub_sleep (grub_uint32_t s)
   grub_millisleep (1000 * s);
 }
 
-void grub_install_get_time_ms (grub_uint64_t (*get_time_ms_func) (void));
-
 #endif /* ! KERNEL_TIME_HEADER */
