@@ -1,6 +1,6 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002,2003,2005,2006,2007,2008,2009,2010  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2003,2005,2006,2007  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,41 +21,38 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <stdarg.h>
 #include <setjmp.h>
 #include <unistd.h>
 
-#include <config.h>
-#include <grub/types.h>
-#include <grub/symbol.h>
-#include <grub/emu/misc.h>
-
-char *grub_util_get_path (const char *dir, const char *file);
-size_t grub_util_get_image_size (const char *path);
-char *grub_util_read_image (const char *path);
-void grub_util_load_image (const char *path, char *buf);
-void grub_util_write_image (const char *img, size_t size, FILE *out,
-			    const char *name);
-void grub_util_write_image_at (const void *img, size_t size, off_t offset,
-			       FILE *out, const char *name);
-
-#ifdef __MINGW32__
-
-#define fseeko fseeko64
-#define ftello ftello64
-
-void sync (void);
-int fsync (int fno);
-
-grub_int64_t grub_util_get_disk_size (char *name);
-
+#ifdef __NetBSD__
+/* NetBSD uses /boot for its boot block.  */
+# define DEFAULT_DIRECTORY	"/grub"
+#else
+# define DEFAULT_DIRECTORY	"/boot/grub"
 #endif
 
+#define DEFAULT_DEVICE_MAP	DEFAULT_DIRECTORY "/device.map"
 
-char *make_system_path_relative_to_its_root (const char *path);
+extern char *progname;
+extern int verbosity;
+extern jmp_buf main_env;
 
-char *canonicalize_file_name (const char *path);
+void grub_util_info (const char *fmt, ...);
+void grub_util_error (const char *fmt, ...) __attribute__ ((noreturn));
 
-void grub_util_init_nls (void);
+void *xmalloc (size_t size);
+void *xrealloc (void *ptr, size_t size);
+char *xstrdup (const char *str);
+
+char *grub_util_get_path (const char *dir, const char *file);
+size_t grub_util_get_fp_size (FILE *fp);
+size_t grub_util_get_image_size (const char *path);
+void grub_util_read_at (void *img, size_t len, off_t offset, FILE *fp);
+char *grub_util_read_image (const char *path);
+void grub_util_load_image (const char *path, char *buf);
+void grub_util_write_image (const char *img, size_t size, FILE *out);
+void grub_util_write_image_at (const void *img, size_t size, off_t offset,
+			       FILE *out);
+char *grub_util_get_disk_name (int disk, char *name);
 
 #endif /* ! GRUB_UTIL_MISC_HEADER */
