@@ -1,21 +1,20 @@
 /* env.c - Environment variables */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003,2005,2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2003,2005,2006,2007,2008  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/env.h>
@@ -54,7 +53,7 @@ grub_env_hashval (const char *s)
 {
   unsigned int i = 0;
 
-  /* XXX: This can be done much more effecient.  */
+  /* XXX: This can be done much more efficiently.  */
   while (*s)
     i += 5 * *(s++);
 
@@ -97,11 +96,14 @@ grub_env_context_open (void)
       for (var = context->prev->vars[i]; var; var = var->next)
 	{
 	  if (var->type == GRUB_ENV_VAR_GLOBAL)
-	    if (grub_env_set (var->name, var->value) != GRUB_ERR_NONE)
-	      {
-		grub_env_context_close ();
-		return grub_errno;
-	      }
+	    {
+	      if (grub_env_set (var->name, var->value) != GRUB_ERR_NONE)
+		{
+		  grub_env_context_close ();
+		  return grub_errno;
+		}
+	      grub_register_variable_hook (var->name, var->read_hook, var->write_hook);
+	    }
 	}
     }
   
@@ -122,7 +124,7 @@ grub_env_context_close (void)
     {
       struct grub_env_var *p, *q;
       
-      for (p = current_context->prev->vars[i]; p; p = q)
+      for (p = current_context->vars[i]; p; p = q)
 	{
 	  q = p->next;
 	  grub_free (p);
@@ -329,12 +331,7 @@ grub_register_variable_hook (const char *name,
 
   if (! var)
     {
-      char *val = grub_strdup ("");
-
-      if (! val)
-	return grub_errno;
-      
-      if (grub_env_set (name, val) != GRUB_ERR_NONE)
+      if (grub_env_set (name, "") != GRUB_ERR_NONE)
 	return grub_errno;
       
       var = grub_env_find (name);
