@@ -26,7 +26,7 @@
 grub_err_t
 grub_arch_dl_check_header (void *ehdr)
 {
-  Elf_Ehdr *e = ehdr;
+  Elf32_Ehdr *e = ehdr;
 
   /* Check the magic numbers.  */
   if (e->e_ident[EI_CLASS] != ELFCLASS32
@@ -42,26 +42,28 @@ grub_arch_dl_check_header (void *ehdr)
 grub_err_t
 grub_arch_dl_relocate_symbols (grub_dl_t mod, void *ehdr)
 {
-  Elf_Ehdr *e = ehdr;
-  Elf_Shdr *s;
-  Elf_Word entsize;
+  Elf32_Ehdr *e = ehdr;
+  Elf32_Shdr *s;
+  Elf32_Sym *symtab;
+  Elf32_Word entsize;
   unsigned i;
-
+  
   /* Find a symbol table.  */
-  for (i = 0, s = (Elf_Shdr *) ((char *) e + e->e_shoff);
+  for (i = 0, s = (Elf32_Shdr *) ((char *) e + e->e_shoff);
        i < e->e_shnum;
-       i++, s = (Elf_Shdr *) ((char *) s + e->e_shentsize))
+       i++, s = (Elf32_Shdr *) ((char *) s + e->e_shentsize))
     if (s->sh_type == SHT_SYMTAB)
       break;
 
   if (i == e->e_shnum)
     return grub_error (GRUB_ERR_BAD_MODULE, "no symtab found");
-
+  
+  symtab = (Elf32_Sym *) ((char *) e + s->sh_offset);
   entsize = s->sh_entsize;
-
-  for (i = 0, s = (Elf_Shdr *) ((char *) e + e->e_shoff);
+  
+  for (i = 0, s = (Elf32_Shdr *) ((char *) e + e->e_shoff);
        i < e->e_shnum;
-       i++, s = (Elf_Shdr *) ((char *) s + e->e_shentsize))
+       i++, s = (Elf32_Shdr *) ((char *) s + e->e_shentsize))
     if (s->sh_type == SHT_RELA)
       {
 	grub_dl_segment_t seg;
@@ -73,64 +75,64 @@ grub_arch_dl_relocate_symbols (grub_dl_t mod, void *ehdr)
 
 	if (seg)
 	  {
-	    Elf_Rela *rel, *max;
-
-	    for (rel = (Elf_Rela *) ((char *) e + s->sh_offset),
+	    Elf32_Rela *rel, *max;
+	    
+	    for (rel = (Elf32_Rela *) ((char *) e + s->sh_offset),
 		   max = rel + s->sh_size / s->sh_entsize;
 		 rel < max;
 		 rel++)
 	      {
-		Elf_Word *addr;
-		Elf_Sym *sym;
+		Elf32_Word *addr;
+		Elf32_Sym *sym;
 		grub_uint32_t value;
-
+		
 		if (seg->size < rel->r_offset)
 		  return grub_error (GRUB_ERR_BAD_MODULE,
 				     "reloc offset is out of the segment");
-
-		addr = (Elf_Word *) ((char *) seg->addr + rel->r_offset);
-		sym = (Elf_Sym *) ((char *) mod->symtab
-				     + entsize * ELF_R_SYM (rel->r_info));
-
+		
+		addr = (Elf32_Word *) ((char *) seg->addr + rel->r_offset);
+		sym = (Elf32_Sym *) ((char *) symtab
+				     + entsize * ELF32_R_SYM (rel->r_info));
+		
 		/* On the PPC the value does not have an explicit
 		   addend, add it.  */
 		value = sym->st_value + rel->r_addend;
-		switch (ELF_R_TYPE (rel->r_info))
+		switch (ELF32_R_TYPE (rel->r_info))
 		  {
 		  case R_PPC_ADDR16_LO:
-		    *(Elf_Half *) addr = value;
+		    *(Elf32_Half *) addr = value;
 		    break;
-
+		    
 		  case R_PPC_REL24:
 		    {
-		      Elf_Sword delta = value - (Elf_Word) addr;
-
+		      Elf32_Sword delta = value - (Elf32_Word) addr;
+		      
 		      if (delta << 6 >> 6 != delta)
 			return grub_error (GRUB_ERR_BAD_MODULE, "Relocation overflow");
 		      *addr = (*addr & 0xfc000003) | (delta & 0x3fffffc);
 		      break;
 		    }
-
+		    
 		  case R_PPC_ADDR16_HA:
-		    *(Elf_Half *) addr = (value + 0x8000) >> 16;
+		    *(Elf32_Half *) addr = (value + 0x8000) >> 16;
 		    break;
-
+		    
 		  case R_PPC_ADDR32:
 		    *addr = value;
 		    break;
-
+		    
 		  case R_PPC_REL32:
-		    *addr = value - (Elf_Word) addr;
+		    *addr = value - (Elf32_Word) addr;
 		    break;
-
+		    
 		  default:
 		    return grub_error (GRUB_ERR_NOT_IMPLEMENTED_YET,
 				       "This relocation (%d) is not implemented yet",
-				       ELF_R_TYPE (rel->r_info));
+				       ELF32_R_TYPE (rel->r_info));
 		  }
 	      }
 	  }
       }
-
+  
   return GRUB_ERR_NONE;
 }
