@@ -1,21 +1,20 @@
 /* rescue.c - rescue mode */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002, 2003, 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2003,2005,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/kernel.h>
@@ -152,7 +151,7 @@ grub_rescue_cmd_cat (int argc, char *argv[])
 	{
 	  unsigned char c = buf[i];
 
-	  if (grub_isprint (c) || grub_isspace (c))
+	  if ((grub_isprint (c) || grub_isspace (c)) && c != '\r')
 	    grub_putchar (c);
 	  else
 	    {
@@ -619,7 +618,7 @@ grub_enter_rescue_mode (void)
   /* First of all, attempt to execute the normal mode.  */
   attempt_normal_mode ();
 
-  grub_printf ("Entering into rescue mode...\n");
+  grub_printf ("Entering rescue mode...\n");
   
   grub_rescue_register_command ("boot", grub_rescue_cmd_boot,
 				"boot an operating system");
@@ -660,6 +659,8 @@ grub_enter_rescue_mode (void)
 
       /* Get a command line.  */
       grub_rescue_get_command_line ("grub rescue> ");
+      if (line[0] == 0)
+	continue;
 
       if (grub_parser_split_cmdline (line, getline, &n, &args) || n < 0)
 	continue;
@@ -672,6 +673,7 @@ grub_enter_rescue_mode (void)
 	  val[0] = 0;
 	  grub_env_set (args[0], val + 1);
 	  val[0] = '=';
+          grub_free (args[0]);
 	  continue;
 	}
 
@@ -680,7 +682,10 @@ grub_enter_rescue_mode (void)
 
       /* If nothing is specified, restart.  */
       if (*name == '\0')
-	continue;
+        {
+          grub_free (args[0]);
+          continue;
+        }
 
       /* Find the command and execute it.  */
       for (cmd = grub_rescue_command_list; cmd; cmd = cmd->next)
@@ -698,5 +703,7 @@ grub_enter_rescue_mode (void)
 	  grub_printf ("Unknown command `%s'\n", name);
 	  grub_printf ("Try `help' for usage\n");
 	}
+
+      grub_free (args[0]);
     }
 }
