@@ -1,6 +1,6 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2002,2003,2005,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 2002,2003,2005,2007,2008,2009  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,12 +22,6 @@
 #include <grub/misc.h>
 #include <grub/env.h>
 
-/* The list of terminals.  */
-static grub_term_t grub_term_list;
-
-/* The current terminal.  */
-static grub_term_t grub_cur_term;
-
 /* The amount of lines counted by the pager.  */
 static int grub_more_lines;
 
@@ -37,58 +31,18 @@ static int grub_more;
 /* The current cursor state.  */
 static int cursor_state = 1;
 
-void
-grub_term_register (grub_term_t term)
-{
-  term->next = grub_term_list;
-  grub_term_list = term;
-}
-
-void
-grub_term_unregister (grub_term_t term)
-{
-  grub_term_t *p, q;
-  
-  for (p = &grub_term_list, q = *p; q; p = &(q->next), q = q->next)
-    if (q == term)
+struct grub_handler_class grub_term_input_class =
       {
-        *p = q->next;
-	break;
-      }
-}
+    .name = "terminal_input"
+  };
 
-void
-grub_term_iterate (int (*hook) (grub_term_t term))
-{
-  grub_term_t p;
-  
-  for (p = grub_term_list; p; p = p->next)
-    if (hook (p))
-      break;
-}
+struct grub_handler_class grub_term_output_class =
+      {
+    .name = "terminal_output"
+  };
 
-grub_err_t
-grub_term_set_current (grub_term_t term)
-{
-  if (grub_cur_term && grub_cur_term->fini)
-    if ((grub_cur_term->fini) () != GRUB_ERR_NONE)
-      return grub_errno;
-
-  if (term->init)
-    if ((term->init) () != GRUB_ERR_NONE)
-      return grub_errno;
-  
-  grub_cur_term = term;
-  grub_cls ();
-  grub_setcursor (grub_getcursor ());
-  return GRUB_ERR_NONE;
-}
-
-grub_term_t
-grub_term_get_current (void)
-{
-  return grub_cur_term;
-}
+#define grub_cur_term_input	grub_term_get_current_input ()
+#define grub_cur_term_output	grub_term_get_current_output ()
 
 /* Put a Unicode character.  */
 void
@@ -96,19 +50,19 @@ grub_putcode (grub_uint32_t code)
 {
   int height = grub_getwh () & 255;
 
-  if (code == '\t' && grub_cur_term->getxy)
+  if (code == '\t' && grub_cur_term_output->getxy)
     {
       int n;
-      
+
       n = 8 - ((grub_getxy () >> 8) & 7);
       while (n--)
 	grub_putcode (' ');
 
       return;
     }
-  
-  (grub_cur_term->putchar) (code);
-  
+
+  (grub_cur_term_output->putchar) (code);
+
   if (code == '\n')
     {
       grub_putcode ('\r');
@@ -127,12 +81,12 @@ grub_putcode (grub_uint32_t code)
 	  grub_setcolorstate (GRUB_TERM_COLOR_STANDARD);
 
 	  key = grub_getkey ();
-	  
+
 	  /* Remove the message.  */
 	  grub_gotoxy (1, height - 1);
 	  grub_printf ("        ");
 	  grub_gotoxy (pos >> 8, pos & 0xFF);
-	  
+
 	  /* Scroll one lines or an entire page, depending on the key.  */
 	  if (key == '\r' || key =='\n')
 	    grub_more_lines--;
@@ -153,8 +107,8 @@ grub_putchar (int c)
   grub_ssize_t ret;
 
   buf[size++] = c;
-  ret = grub_utf8_to_ucs4 (&code, buf, size);
-  
+  ret = grub_utf8_to_ucs4 (&code, 1, buf, size, 0);
+
   if (ret > 0)
     {
       size = 0;
@@ -171,70 +125,70 @@ grub_putchar (int c)
 grub_ssize_t
 grub_getcharwidth (grub_uint32_t code)
 {
-  return (grub_cur_term->getcharwidth) (code);
+  return (grub_cur_term_output->getcharwidth) (code);
 }
 
 int
 grub_getkey (void)
 {
-  return (grub_cur_term->getkey) ();
+  return (grub_cur_term_input->getkey) ();
 }
 
 int
 grub_checkkey (void)
 {
-  return (grub_cur_term->checkkey) ();
+  return (grub_cur_term_input->checkkey) ();
 }
 
 grub_uint16_t
 grub_getxy (void)
 {
-  return (grub_cur_term->getxy) ();
+  return (grub_cur_term_output->getxy) ();
 }
 
 grub_uint16_t
 grub_getwh (void)
 {
-  return (grub_cur_term->getwh) ();
+  return (grub_cur_term_output->getwh) ();
 }
 
 void
 grub_gotoxy (grub_uint8_t x, grub_uint8_t y)
 {
-  (grub_cur_term->gotoxy) (x, y);
+  (grub_cur_term_output->gotoxy) (x, y);
 }
 
 void
 grub_cls (void)
 {
-  if ((grub_cur_term->flags & GRUB_TERM_DUMB) || (grub_env_get ("debug")))
+  if ((grub_cur_term_output->flags & GRUB_TERM_DUMB) || (grub_env_get ("debug")))
     {
       grub_putchar ('\n');
       grub_refresh ();
     }
   else
-    (grub_cur_term->cls) ();
+    (grub_cur_term_output->cls) ();
 }
 
 void
 grub_setcolorstate (grub_term_color_state state)
 {
-  if (grub_cur_term->setcolorstate)
-    (grub_cur_term->setcolorstate) (state);
+  if (grub_cur_term_output->setcolorstate)
+    (grub_cur_term_output->setcolorstate) (state);
 }
 
 void
 grub_setcolor (grub_uint8_t normal_color, grub_uint8_t highlight_color)
 {
-  if (grub_cur_term->setcolor)
-    (grub_cur_term->setcolor) (normal_color, highlight_color);
+  if (grub_cur_term_output->setcolor)
+    (grub_cur_term_output->setcolor) (normal_color, highlight_color);
 }
 
 void
 grub_getcolor (grub_uint8_t *normal_color, grub_uint8_t *highlight_color)
 {
-  if (grub_cur_term->getcolor)
-    (grub_cur_term->getcolor) (normal_color, highlight_color);
+  if (grub_cur_term_output->getcolor)
+    (grub_cur_term_output->getcolor) (normal_color, highlight_color);
 }
 
 int
@@ -242,12 +196,12 @@ grub_setcursor (int on)
 {
   int ret = cursor_state;
 
-  if (grub_cur_term->setcursor)
+  if (grub_cur_term_output->setcursor)
     {
-      (grub_cur_term->setcursor) (on);
+      (grub_cur_term_output->setcursor) (on);
       cursor_state = on;
     }
-  
+
   return ret;
 }
 
@@ -260,8 +214,8 @@ grub_getcursor (void)
 void
 grub_refresh (void)
 {
-  if (grub_cur_term->refresh)
-    (grub_cur_term->refresh) ();
+  if (grub_cur_term_output->refresh)
+    (grub_cur_term_output->refresh) ();
 }
 
 void
