@@ -1,21 +1,20 @@
 /* completion.c - complete a command, a disk, a partition or a file */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005  Free Software Foundation, Inc.
+ *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/normal.h>
@@ -139,7 +138,7 @@ iterate_dir (const char *filename, int dir)
       if (add_completion (filename, prefix, GRUB_COMPLETION_TYPE_FILE))
 	return 1;
     }
-  else
+  else if (grub_strcmp (filename, ".") && grub_strcmp (filename, ".."))
     {
       char fname[grub_strlen (filename) + 2];
 
@@ -474,18 +473,19 @@ grub_normal_do_completion (char *buf, int *restore,
       if (num_found == 1)
 	grub_strcat (ret, suffix);
       
-      grub_free (match);
-
       if (*ret == '\0')
 	{
 	  grub_free (ret);
-	  return 0;
+          goto fail;
 	}
       
+      grub_free (argv[0]);
+      grub_free (match);
       return ret;
     }
 
  fail:
+  grub_free (argv[0]);
   grub_free (match);
   grub_errno = GRUB_ERR_NONE;
 
