@@ -1,20 +1,19 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2005,2006,2007,2008  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/normal.h>
@@ -22,6 +21,7 @@
 #include <grub/misc.h>
 #include <grub/mm.h>
 #include <grub/loader.h>
+#include <grub/script.h>
 
 enum update_mode
   {
@@ -335,7 +335,7 @@ insert_string (struct screen *screen, char *s, int update)
 	  int size;
 	  int orig_num, new_num;
 
-	  /* Find a string delimitted by LF.  */
+	  /* Find a string delimited by LF.  */
 	  p = grub_strchr (s, '\n');
 	  if (! p)
 	    p = s + grub_strlen (s);
@@ -1006,7 +1006,7 @@ run (struct screen *screen)
 
 
   /* Execute the script, line for line.  */
-  while (currline < screen->num_lines - 1)
+  while (currline < screen->num_lines)
     {
       editor_getline (&nextline);
       parsed_script = grub_script_parse (nextline, editor_getline);
@@ -1017,23 +1017,20 @@ run (struct screen *screen)
 	  
 	  /* The parsed script was executed, throw it away.  */
 	  grub_script_free (parsed_script);
-
-	  if (grub_errno == GRUB_ERR_NONE && grub_loader_is_loaded ())
-	    /* Implicit execution of boot, only if something is loaded.  */
-	    grub_command_execute ("boot", 0);
 	}
       else
 	break;
     }
 
+  if (grub_errno == GRUB_ERR_NONE && grub_loader_is_loaded ())
+    /* Implicit execution of boot, only if something is loaded.  */
+    grub_command_execute ("boot", 0);
+
   if (grub_errno != GRUB_ERR_NONE)
     {
       grub_print_error ();
       grub_errno = GRUB_ERR_NONE;
-      /* Wait until the user pushes any key so that the user
-	 can see what happened.  */
-      grub_printf ("\nPress any key to continue...");
-      (void) grub_getkey ();
+      grub_wait_after_message ();
     }
 
   return 1;
