@@ -1,21 +1,20 @@
 /* script.c -- Functions to create an in memory description of the script. */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005, 2006  Free Software Foundation, Inc.
+ *  Copyright (C) 2005,2006,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/misc.h>
@@ -25,14 +24,14 @@
 
 /* It is not possible to deallocate the memory when a syntax error was
    found.  Because of that it is required to keep track of all memory
-   allocations.  The memory is free'ed in case of an error, or
+   allocations.  The memory is freed in case of an error, or
    assigned to the parsed script when parsing was successful.  */
 
 /* XXX */
 
 /* In case of the normal malloc, some additional bytes are allocated
    for this datastructure.  All reserved memory is stored in a linked
-   list so it can be easily free'ed.  The original memory can be found
+   list so it can be easily freed.  The original memory can be found
    from &mem.  */
 struct grub_script_mem
 {
@@ -214,6 +213,11 @@ grub_script_create_cmdmenu (struct grub_parser_param *state,
   struct grub_script_cmd_menuentry *cmd;
   int i;
 
+  /* Skip leading newlines to make the sourcecode better readable when
+     using the editor.  */
+  while (*sourcecode == '\n')
+    sourcecode++;
+
   /* Having trailing returns can some some annoying conflicts, remove
      them.  XXX: Can the parser be improved to handle this?  */
   for (i = grub_strlen (sourcecode) - 1; i > 0; i--)
@@ -226,6 +230,7 @@ grub_script_create_cmdmenu (struct grub_parser_param *state,
   cmd = grub_script_malloc (state, sizeof (*cmd));
   cmd->cmd.exec = grub_script_execute_menuentry;
   cmd->cmd.next = 0;
+  /* XXX: Check if this memory is properly freed.  */
   cmd->sourcecode = sourcecode;
   cmd->title = title;
   cmd->options = options;
@@ -292,7 +297,6 @@ struct grub_script *
 grub_script_parse (char *script, grub_err_t (*getline) (char **))
 {
   struct grub_script *parsed;
-  struct grub_script_mem *memfree;
   struct grub_script_mem *membackup;
   struct grub_lexer_param *lexstate;
   struct grub_parser_param *parsestate;
