@@ -1,20 +1,19 @@
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2005,2007  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/machine/memory.h>
@@ -55,7 +54,7 @@ struct grub_colored_char
 
 struct grub_virtual_screen
 {
-  /* Dimensions of the virual screen.  */
+  /* Dimensions of the virtual screen.  */
   grub_uint32_t width;
   grub_uint32_t height;
 
@@ -78,7 +77,7 @@ struct grub_virtual_screen
   grub_uint8_t fg_color;
   grub_uint8_t bg_color;
 
-  /* Text buffer for virual screen. Contains (columns * rows) number
+  /* Text buffer for virtual screen. Contains (columns * rows) number
      of entries.  */
   struct grub_colored_char *text_buffer;
 };
@@ -377,7 +376,7 @@ scroll_up (void)
       virtual_screen.text_buffer[i].index = 0;
     }
 
-  /* Scroll frambuffer with one line to up.  */
+  /* Scroll framebuffer with one line to up.  */
   grub_memmove (framebuffer,
                 framebuffer
                 + bytes_per_scan_line * virtual_screen.char_height,
@@ -565,13 +564,6 @@ grub_virtual_screen_setcolorstate (grub_term_color_state state)
     }
 }
 
-static void
-grub_virtual_screen_setcolor (grub_uint8_t normal_color __attribute__ ((unused)),
-			      grub_uint8_t highlight_color __attribute__ ((unused)))
-{
-  /* FIXME */
-}
-
 static void
 grub_vesafb_setcursor (int on)
 {
@@ -600,7 +592,6 @@ static struct grub_term grub_vesafb_term =
     .gotoxy = grub_vesafb_gotoxy,
     .cls = grub_vesafb_cls,
     .setcolorstate = grub_virtual_screen_setcolorstate,
-    .setcolor = grub_virtual_screen_setcolor,
     .setcursor = grub_vesafb_setcursor,
     .flags = 0,
     .next = 0
