@@ -1,21 +1,20 @@
 /*  ofconsole.c -- Open Firmware console for GRUB.  */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+ *  Copyright (C) 2003,2004,2005,2007,2008  Free Software Foundation, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/term.h>
@@ -34,8 +33,10 @@ static grub_uint8_t grub_ofconsole_height;
 static int grub_curr_x;
 static int grub_curr_y;
 
+#ifndef __i386__
 static int grub_keybuf;
 static int grub_buflen;
+#endif
 
 struct color
 {
@@ -57,13 +58,16 @@ static struct color colors[8] =
     { MAX, MAX, MAX}
   };
 
-static int fgcolor = 7;
-static int bgcolor = 0;
+static grub_uint8_t grub_ofconsole_normal_color = 0x7;
+static grub_uint8_t grub_ofconsole_highlight_color = 0x70;
 
 /* Write control characters to the console.  */
 static void
 grub_ofconsole_writeesc (const char *str)
 {
+  if (grub_ieee1275_test_flag (GRUB_IEEE1275_FLAG_NO_ANSI))
+    return;
+
   while (*str)
     {
       char chr = *(str++);
@@ -85,7 +89,10 @@ grub_ofconsole_putchar (grub_uint32_t c)
     {
       grub_curr_x++;
       if (grub_curr_x > grub_ofconsole_width)
-	grub_putcode ('\n');
+        {
+          grub_putcode ('\n');
+          grub_curr_x++;
+        }
     }
   grub_ieee1275_write (stdout_ihandle, &chr, 1, 0);
 }
@@ -107,12 +114,12 @@ grub_ofconsole_setcolorstate (grub_term_color_state state)
     {
     case GRUB_TERM_COLOR_STANDARD:
     case GRUB_TERM_COLOR_NORMAL:
-      fg = fgcolor;
-      bg = bgcolor;
+      fg = grub_ofconsole_normal_color & 0x0f;
+      bg = grub_ofconsole_normal_color >> 4;
       break;
     case GRUB_TERM_COLOR_HIGHLIGHT:
-      fg = bgcolor;
-      bg = fgcolor;
+      fg = grub_ofconsole_highlight_color & 0x0f;
+      bg = grub_ofconsole_highlight_color >> 4;
       break;
     default:
       return;
@@ -126,10 +133,18 @@ static void
 grub_ofconsole_setcolor (grub_uint8_t normal_color,
 			 grub_uint8_t highlight_color)
 {
-  fgcolor = normal_color;
-  bgcolor = highlight_color;
+  grub_ofconsole_normal_color = normal_color;
+  grub_ofconsole_highlight_color = highlight_color;
 }
 
+static void
+grub_ofconsole_getcolor (grub_uint8_t *normal_color, grub_uint8_t *highlight_color)
+{
+  *normal_color = grub_ofconsole_normal_color;
+  *highlight_color = grub_ofconsole_highlight_color;
+}
+
+#ifndef __i386__
 static int
 grub_ofconsole_readkey (int *key)
 {
@@ -214,6 +229,7 @@ grub_ofconsole_getkey (void)
   
   return key;
 }
+#endif
 
 static grub_uint16_t
 grub_ofconsole_getxy (void)
@@ -275,11 +291,28 @@ static void
 grub_ofconsole_gotoxy (grub_uint8_t x, grub_uint8_t y)
 {
   char s[11]; /* 5 + 3 + 3.  */
-  grub_curr_x = x;
-  grub_curr_y = y;
 
-  grub_sprintf (s, "\e[%d;%dH", y + 1, x + 1);
-  grub_ofconsole_writeesc (s);
+  if (! grub_ieee1275_test_flag (GRUB_IEEE1275_FLAG_NO_ANSI))
+    {
+      grub_curr_x = x;
+      grub_curr_y = y;
+
+      grub_sprintf (s, "\e[%d;%dH", y + 1, x + 1);
+      grub_ofconsole_writeesc (s);
+    }
+  else
+    {
+      if ((y == grub_curr_y) && (x == grub_curr_x - 1))
+        {
+          char chr;
+
+          chr = '\b';
+          grub_ieee1275_write (stdout_ihandle, &chr, 1, 0);
+        }
+
+      grub_curr_x = x;
+      grub_curr_y = y;
+    }
 }
 
 static void
@@ -293,9 +326,13 @@ grub_ofconsole_cls (void)
 }
 
 static void
-grub_ofconsole_setcursor (int on __attribute ((unused)))
+grub_ofconsole_setcursor (int on)
 {
-  /* XXX: Not supported.  */
+  /* Understood by the Open Firmware flavour in OLPC.  */
+  if (on)
+    grub_ieee1275_interpret ("cursor-on", 0);
+  else
+    grub_ieee1275_interpret ("cursor-off", 0);
 }
 
 static void
@@ -307,35 +344,35 @@ grub_ofconsole_refresh (void)
 static grub_err_t
 grub_ofconsole_init (void)
 {
-  unsigned char data[4];
   grub_ssize_t actual;
   int col;
 
   /* The latest PowerMacs don't actually initialize the screen for us, so we
-   * use this trick to re-open the output device.  */
-  grub_ieee1275_interpret ("output-device output", 0);
+   * use this trick to re-open the output device (but we avoid doing this on
+   * platforms where it's known to be broken). */
+  if (! grub_ieee1275_test_flag (GRUB_IEEE1275_FLAG_BROKEN_OUTPUT))
+    grub_ieee1275_interpret ("output-device output", 0);
 
-  if (grub_ieee1275_get_property (grub_ieee1275_chosen, "stdout", data,
-				  sizeof data, &actual)
-      || actual != sizeof data)
+  if (grub_ieee1275_get_integer_property (grub_ieee1275_chosen, "stdout", &stdout_ihandle,
+					  sizeof stdout_ihandle, &actual)
+      || actual != sizeof stdout_ihandle)
     return grub_error (GRUB_ERR_UNKNOWN_DEVICE, "Cannot find stdout");
 
-  stdout_ihandle = grub_ieee1275_decode_int_4 (data);
-  
-  if (grub_ieee1275_get_property (grub_ieee1275_chosen, "stdin", data,
-				  sizeof data, &actual)
-      || actual != sizeof data)
+  if (grub_ieee1275_get_integer_property (grub_ieee1275_chosen, "stdin", &stdin_ihandle,
+					  sizeof stdin_ihandle, &actual)
+      || actual != sizeof stdin_ihandle)
     return grub_error (GRUB_ERR_UNKNOWN_DEVICE, "Cannot find stdin");
 
-  stdin_ihandle = grub_ieee1275_decode_int_4 (data);
-
   /* Initialize colors.  */
-  for (col = 0; col < 7; col++)
-    grub_ieee1275_set_color (stdout_ihandle, col, colors[col].red,
-			     colors[col].green, colors[col].blue);
+  if (! grub_ieee1275_test_flag (GRUB_IEEE1275_FLAG_CANNOT_SET_COLORS))
+    {
+      for (col = 0; col < 7; col++)
+	grub_ieee1275_set_color (stdout_ihandle, col, colors[col].red,
+				 colors[col].green, colors[col].blue);
 
-  /* Set the right fg and bg colors.  */
-  grub_ofconsole_setcolorstate (GRUB_TERM_COLOR_NORMAL);
+    /* Set the right fg and bg colors.  */
+      grub_ofconsole_setcolorstate (GRUB_TERM_COLOR_NORMAL);
+    }
 
   return 0;
 }
@@ -355,14 +392,20 @@ static struct grub_term grub_ofconsole_term =
     .fini = grub_ofconsole_fini,
     .putchar = grub_ofconsole_putchar,
     .getcharwidth = grub_ofconsole_getcharwidth,
+#ifdef __i386__
+    .checkkey = grub_console_checkkey,
+    .getkey = grub_console_getkey,
+#else
     .checkkey = grub_ofconsole_checkkey,
     .getkey = grub_ofconsole_getkey,
+#endif
     .getxy = grub_ofconsole_getxy,
     .getwh = grub_ofconsole_getwh,
     .gotoxy = grub_ofconsole_gotoxy,
     .cls = grub_ofconsole_cls,
     .setcolorstate = grub_ofconsole_setcolorstate,
     .setcolor = grub_ofconsole_setcolor,
+    .getcolor = grub_ofconsole_getcolor,
     .setcursor = grub_ofconsole_setcursor,
     .refresh = grub_ofconsole_refresh,
     .flags = 0,
