@@ -1,20 +1,30 @@
-/*
- *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1998-2003,2004,2005 Free Software Foundation, Inc.
- *
- *  GRUB is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  GRUB is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
- */
+/****************************************************************************
+ * Copyright (c) 1998-2003,2004,2005 Free Software Foundation, Inc.         *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
 
 /**********************************************************************
  * This code is a modification of lib_tparm.c found in ncurses-5.2. The
@@ -66,7 +76,7 @@ typedef char grub_bool_t;
 
 #define UChar(c) ((unsigned char)(c))
 
-//MODULE_ID("$Id$")
+//MODULE_ID("$Id: tparm.c,v 1.3 2006/10/14 15:24:53 okuji Exp $")
 
 /*
  *	char *
@@ -167,7 +177,7 @@ save_text(const char *fmt, const char *s, int len)
 
     get_space(s_len + 1);
 
-    (void) grub_snprintf(out_buff + out_used, s_len + 1, fmt, s);
+    (void) grub_sprintf(out_buff + out_used, fmt, s);
     out_used += grub_strlen(out_buff + out_used);
 }
 
@@ -179,7 +189,7 @@ save_number(const char *fmt, int number, int len)
 
     get_space((unsigned) len + 1);
 
-    (void) grub_snprintf(out_buff + out_used, len + 1, fmt, number);
+    (void) grub_sprintf(out_buff + out_used, fmt, number);
     out_used += grub_strlen(out_buff + out_used);
 }
 
@@ -351,8 +361,6 @@ analyze(const char *string, char *p_is_s[NUM_PARM], int *popcount)
     const char *cp = string;
     static char dummy[] = "";
 
-    *popcount = 0;
-
     if (cp == 0)
 	return 0;
 
@@ -363,6 +371,7 @@ analyze(const char *string, char *p_is_s[NUM_PARM], int *popcount)
     }
 
     grub_memset(p_is_s, 0, sizeof(p_is_s[0]) * NUM_PARM);
+    *popcount = 0;
 
     while ((cp - string) < (int) len2) {
 	if (*cp == '%') {
