@@ -1,21 +1,20 @@
 /* biosdisk.c - emulate biosdisk */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1999,2000,2001,2002,2003,2004,2006,2007  Free Software Foundation, Inc.
+ *  Copyright (C) 1999,2000,2001,2002,2003,2004,2006,2007,2008  Free Software Foundation, Inc.
  *
- *  GRUB is free software; you can redistribute it and/or modify
+ *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  GRUB is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with GRUB; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <grub/disk.h>
@@ -24,6 +23,7 @@
 #include <grub/types.h>
 #include <grub/err.h>
 #include <grub/util/misc.h>
+#include <grub/util/biosdisk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -77,7 +77,19 @@ struct hd_geometry
 # endif /* ! LOOP_MAJOR */
 #endif /* __linux__ */
 
-static char *map[256];
+#ifdef __CYGWIN__
+# include <sys/ioctl.h>
+# include <cygwin/fs.h> /* BLKGETSIZE64 */
+# include <cygwin/hdreg.h> /* HDIO_GETGEO */
+# define MAJOR(dev)	((unsigned) ((dev) >> 16))
+# define FLOPPY_MAJOR	2
+#endif
+
+struct
+{
+  char *drive;
+  char *device;
+} map[256];
 
 #ifdef __linux__
 /* Check if we have devfs support.  */
@@ -98,47 +110,33 @@ have_devfs (void)
 #endif /* __linux__ */
 
 static int
-get_drive (const char *name)
+find_grub_drive (const char *name)
 {
-  unsigned long drive;
-  char *p;
-  
-  if ((name[0] != 'f' && name[0] != 'h') || name[1] != 'd')
-    goto fail;
+  unsigned int i;
 
-  drive = strtoul (name + 2, &p, 10);
-  if (p == name + 2)
-    goto fail;
+  if (name)
+    {
+      for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
+	if (map[i].drive && ! strcmp (map[i].drive, name))
+	  return i;
+    }
+  else
+    {
+      for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
+	if (! map[i].drive)
+	  return i;
+    }
 
-  if (name[0] == 'h')
-    drive += 0x80;
-
-  if (drive > sizeof (map) / sizeof (map[0]))
-    goto fail;
-  
-  return (int) drive;
-
- fail:
-  grub_error (GRUB_ERR_UNKNOWN_DEVICE, "not a biosdisk");
   return -1;
 }
 
-static int
-call_hook (int (*hook) (const char *name), int drive)
-{
-  char name[10];
-
-  sprintf (name, (drive & 0x80) ? "hd%d" : "fd%d", drive & (~0x80));
-  return hook (name);
-}
-
 static int
 grub_util_biosdisk_iterate (int (*hook) (const char *name))
 {
   unsigned i;
 
   for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
-    if (map[i] && call_hook (hook, i))
+    if (map[i].drive && hook (map[i].drive))
       return 1;
 
   return 0;
@@ -150,26 +148,23 @@ grub_util_biosdisk_open (const char *name, grub_disk_t disk)
   int drive;
   struct stat st;
   
-  drive = get_drive (name);
+  drive = find_grub_drive (name);
   if (drive < 0)
-    return grub_errno;
-
-  if (! map[drive])
     return grub_error (GRUB_ERR_BAD_DEVICE,
 		       "no mapping exists for `%s'", name);
   
-  disk->has_partitions = (drive & 0x80);
+  disk->has_partitions = 1;
   disk->id = drive;
 
   /* Get the size.  */
-#ifdef __linux__
+#if defined(__linux__) || defined(__CYGWIN__)
   {
     unsigned long long nr;
     int fd;
 
-    fd = open (map[drive], O_RDONLY);
-    if (! fd)
-      return grub_error (GRUB_ERR_BAD_DEVICE, "cannot open `%s'", map[drive]);
+    fd = open (map[drive].device, O_RDONLY);
+    if (fd == -1)
+      return grub_error (GRUB_ERR_BAD_DEVICE, "cannot open `%s'", map[drive].device);
 
     if (fstat (fd, &st) < 0 || ! S_ISBLK (st.st_mode))
       {
@@ -199,8 +194,8 @@ grub_util_biosdisk_open (const char *name, grub_disk_t disk)
 #elif !defined (__GNU__)
 # warning "No special routine to get the size of a block device is implemented for your OS. This is not possibly fatal."
 #endif
-  if (stat (map[drive], &st) < 0)
-    return grub_error (GRUB_ERR_BAD_DEVICE, "cannot stat `%s'", map[drive]);
+  if (stat (map[drive].device, &st) < 0)
+    return grub_error (GRUB_ERR_BAD_DEVICE, "cannot stat `%s'", map[drive].device);
 
   disk->total_sectors = st.st_size >> GRUB_DISK_SECTOR_BITS;
   
@@ -220,42 +215,31 @@ linux_find_partition (char *dev, unsigned long sector)
   char real_dev[PATH_MAX];
 
   strcpy(real_dev, dev);
-  
+
   if (have_devfs () && strcmp (real_dev + len - 5, "/disc") == 0)
     {
       p = real_dev + len - 4;
       format = "part%d";
     }
-  else if ((strncmp (real_dev + 5, "hd", 2) == 0
-	    || strncmp (real_dev + 5, "sd", 2) == 0)
-	   && real_dev[7] >= 'a' && real_dev[7] <= 'z')
+  else if (real_dev[len - 1] >= '0' && real_dev[len - 1] <= '9')
     {
-      p = real_dev + 8;
-      format = "%d";
-    }
-  else if (strncmp (real_dev + 5, "rd/c", 4) == 0)
-    {
-      p = strchr (real_dev + 9, 'd');
-      if (! p)
-	return 0;
-
-      p++;
-      while (*p && isdigit (*p))
-	p++;
-
+      p = real_dev + len;
       format = "p%d";
     }
   else
-    return 0;
+    {
+      p = real_dev + len;
+      format = "%d";
+    }
 
   for (i = 1; i < 10000; i++)
     {
       int fd;
       struct hd_geometry hdg;
-      
+
       sprintf (p, format, i);
       fd = open (real_dev, O_RDONLY);
-      if (! fd)
+      if (fd == -1)
 	return 0;
 
       if (ioctl (fd, HDIO_GETGEO, &hdg))
@@ -265,7 +249,7 @@ linux_find_partition (char *dev, unsigned long sector)
 	}
 
       close (fd);
-      
+
       if (hdg.start == sector)
 	{
 	  strcpy (dev, real_dev);
@@ -299,8 +283,8 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
     int is_partition = 0;
     char dev[PATH_MAX];
     
-    strcpy (dev, map[disk->id]);
-    if (disk->partition && strncmp (map[disk->id], "/dev/", 5) == 0)
+    strcpy (dev, map[disk->id].device);
+    if (disk->partition && strncmp (map[disk->id].device, "/dev/", 5) == 0)
       is_partition = linux_find_partition (dev, disk->partition->start);
     
     /* Open the partition.  */
@@ -314,15 +298,15 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
 
     /* Make the buffer cache consistent with the physical disk.  */
     ioctl (fd, BLKFLSBUF, 0);
-    
+
     if (is_partition)
       sector -= disk->partition->start;
   }
 #else /* ! __linux__ */
-  fd = open (map[disk->id], flags);
+  fd = open (map[disk->id].device, flags);
   if (fd < 0)
     {
-      grub_error (GRUB_ERR_BAD_DEVICE, "cannot open `%s'", map[disk->id]);
+      grub_error (GRUB_ERR_BAD_DEVICE, "cannot open `%s'", map[disk->id].device);
       return -1;
     }
 #endif /* ! __linux__ */
@@ -340,7 +324,7 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
     offset = (loff_t) sector << GRUB_DISK_SECTOR_BITS;
     if (_llseek (fd, offset >> 32, offset & 0xffffffff, &result, SEEK_SET))
       {
-	grub_error (GRUB_ERR_BAD_DEVICE, "cannot seek `%s'", map[disk->id]);
+	grub_error (GRUB_ERR_BAD_DEVICE, "cannot seek `%s'", map[disk->id].device);
 	close (fd);
 	return -1;
       }
@@ -351,7 +335,7 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
 
     if (lseek (fd, offset, SEEK_SET) != offset)
       {
-	grub_error (GRUB_ERR_BAD_DEVICE, "cannot seek `%s'", map[disk->id]);
+	grub_error (GRUB_ERR_BAD_DEVICE, "cannot seek `%s'", map[disk->id].device);
 	close (fd);
 	return -1;
       }
@@ -426,13 +410,13 @@ grub_util_biosdisk_read (grub_disk_t disk, grub_disk_addr_t sector,
 #ifdef __linux__
   if (sector == 0 && size > 1)
     {
-      /* Work around a bug in linux's ez remapping.  Linux remaps all
+      /* Work around a bug in Linux ez remapping.  Linux remaps all
 	 sectors that are read together with the MBR in one read.  It
 	 should only remap the MBR, so we split the read in two 
 	 parts. -jochen  */
       if (nread (fd, buf, GRUB_DISK_SECTOR_SIZE) != GRUB_DISK_SECTOR_SIZE)
 	{
-	  grub_error (GRUB_ERR_READ_ERROR, "cannot read `%s'", map[disk->id]);
+	  grub_error (GRUB_ERR_READ_ERROR, "cannot read `%s'", map[disk->id].device);
 	  close (fd);
 	  return grub_errno;
 	}
@@ -444,7 +428,7 @@ grub_util_biosdisk_read (grub_disk_t disk, grub_disk_addr_t sector,
   
   if (nread (fd, buf, size << GRUB_DISK_SECTOR_BITS)
       != (ssize_t) (size << GRUB_DISK_SECTOR_BITS))
-    grub_error (GRUB_ERR_READ_ERROR, "cannot read from `%s'", map[disk->id]);
+    grub_error (GRUB_ERR_READ_ERROR, "cannot read from `%s'", map[disk->id].device);
 
   close (fd);
   return grub_errno;
@@ -462,7 +446,7 @@ grub_util_biosdisk_write (grub_disk_t disk, grub_disk_addr_t sector,
   
   if (nwrite (fd, buf, size << GRUB_DISK_SECTOR_BITS)
       != (ssize_t) (size << GRUB_DISK_SECTOR_BITS))
-    grub_error (GRUB_ERR_WRITE_ERROR, "cannot write to `%s'", map[disk->id]);
+    grub_error (GRUB_ERR_WRITE_ERROR, "cannot write to `%s'", map[disk->id].device);
 
   close (fd);
   return grub_errno;
@@ -486,8 +470,9 @@ read_device_map (const char *dev_map)
   FILE *fp;
   char buf[1024];	/* XXX */
   int lineno = 0;
-  auto void show_error (const char *msg);
+  struct stat st;
 
+  auto void show_error (const char *msg);
   void show_error (const char *msg)
     {
       grub_util_error ("%s:%d: %s", dev_map, lineno, msg);
@@ -517,14 +502,20 @@ read_device_map (const char *dev_map)
 	show_error ("No open parenthesis found");
 
       p++;
-      drive = get_drive (p);
-      if (drive < 0 || drive >= (int) (sizeof (map) / sizeof (map[0])))
-	show_error ("Bad device name");
+      /* Find a free slot.  */
+      drive = find_grub_drive (NULL);
+      if (drive < 0)
+	show_error ("Map table size exceeded");
 
+      e = p;
       p = strchr (p, ')');
       if (! p)
 	show_error ("No close parenthesis found");
 
+      map[drive].drive = xmalloc (p - e + sizeof ('\0'));
+      strncpy (map[drive].drive, e, p - e + sizeof ('\0'));
+      map[drive].drive[p - e] = '\0';
+
       p++;
       /* Skip leading spaces.  */
       while (*p && isspace (*p))
@@ -539,19 +530,21 @@ read_device_map (const char *dev_map)
 	e++;
       *e = '\0';
 
-      /* Multiple entries for a given drive is not allowed.  */
-      if (map[drive])
-	show_error ("Duplicated entry found");
+      if (stat (p, &st) == -1)
+	{
+	  grub_util_info ("Cannot stat `%s', skipping", p);
+	  continue;
+	}
 
 #ifdef __linux__
       /* On Linux, the devfs uses symbolic links horribly, and that
 	 confuses the interface very much, so use realpath to expand
 	 symbolic links.  */
-      map[drive] = xmalloc (PATH_MAX);
-      if (! realpath (p, map[drive]))
+      map[drive].device = xmalloc (PATH_MAX);
+      if (! realpath (p, map[drive].device))
 	grub_util_error ("Cannot get the real path of `%s'", p);
 #else
-      map[drive] = xstrdup (p);
+      map[drive].device = xstrdup (p);
 #endif
     }
 
@@ -571,7 +564,13 @@ grub_util_biosdisk_fini (void)
   unsigned i;
   
   for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
-    free (map[i]);
+    {
+      if (map[i].drive)
+	free (map[i].drive);
+      if (map[i].device)
+	free (map[i].device);
+      map[i].drive = map[i].device = NULL;
+    }
   
   grub_disk_dev_unregister (&grub_util_biosdisk_dev);
 }
@@ -582,7 +581,7 @@ make_device_name (int drive, int dos_part, int bsd_part)
   char *p;
 
   p = xmalloc (30);
-  sprintf (p, (drive & 0x80) ? "hd%d" : "fd%d", drive & ~0x80);
+  sprintf (p, "%s", map[drive].drive);
   
   if (dos_part >= 0)
     sprintf (p + strlen (p), ",%d", dos_part + 1);
@@ -594,40 +593,35 @@ make_device_name (int drive, int dos_part, int bsd_part)
 }
 
 static char *
-get_os_disk (const char *os_dev)
+convert_system_partition_to_system_disk (const char *os_dev)
 {
-  char *path, *p;
-  
 #if defined(__linux__)
-  path = xmalloc (PATH_MAX);
+  char *path = xmalloc (PATH_MAX);
   if (! realpath (os_dev, path))
     return 0;
   
   if (strncmp ("/dev/", path, 5) == 0)
     {
-      p = path + 5;
+      char *p = path + 5;
 
-      if (have_devfs ())
+      /* If this is an IDE disk.  */
+      if (strncmp ("ide/", p, 4) == 0)
 	{
-	  /* If this is an IDE disk.  */
-	  if (strncmp ("/dev/ide/", p, 9) == 0)
-	    {
-	      p = strstr (p, "part");
-	      if (p)
-		strcpy (p, "disc");
-
-	      return path;
-	    }
-
-	  /* If this is a SCSI disk.  */
-	  if (strncmp ("/dev/scsi/", p, 10) == 0)
-	    {
-	      p = strstr (p, "part");
-	      if (p)
-		strcpy (p, "disc");
-
-	      return path;
-	    }
+	  p = strstr (p, "part");
+	  if (p)
+	    strcpy (p, "disc");
+	  
+	  return path;
+	}
+      
+      /* If this is a SCSI disk.  */
+      if (strncmp ("scsi/", p, 5) == 0)
+	{
+	  p = strstr (p, "part");
+	  if (p)
+	    strcpy (p, "disc");
+	  
+	  return path;
 	}
       
       /* If this is a DAC960 disk.  */
@@ -641,47 +635,103 @@ get_os_disk (const char *os_dev)
 	  return path;
 	}
       
-      /* If this is an IDE disk or a SCSI disk.  */
+      /* If this is a CCISS disk.  */
+      if (strncmp ("cciss/c", p, sizeof ("cciss/c") - 1) == 0)
+	{
+	  /* /dev/cciss/c[0-9]+d[0-9]+(p[0-9]+)? */
+	  p = strchr (p, 'p');
+	  if (p)
+	    *p = '\0';
+
+	  return path;
+	}
+      
+      /* If this is a Compaq Intelligent Drive Array.  */
+      if (strncmp ("ida/c", p, sizeof ("ida/c") - 1) == 0)
+	{
+	  /* /dev/ida/c[0-9]+d[0-9]+(p[0-9]+)? */
+	  p = strchr (p, 'p');
+	  if (p)
+	    *p = '\0';
+
+	  return path;
+	}
+      
+      /* If this is an I2O disk.  */
+      if (strncmp ("i2o/hd", p, sizeof ("i2o/hd") - 1) == 0)
+      	{
+	  /* /dev/i2o/hd[a-z]([0-9]+)? */
+	  p[sizeof ("i2o/hda") - 1] = '\0';
+	  return path;
+	}
+      
+      /* If this is a MultiMediaCard (MMC).  */
+      if (strncmp ("mmcblk", p, sizeof ("mmcblk") - 1) == 0)
+	{
+	  /* /dev/mmcblk[0-9]+(p[0-9]+)? */
+	  p = strchr (p, 'p');
+	  if (p)
+	    *p = '\0';
+
+	  return path;
+	}
+      
+      /* If this is an IDE, SCSI or Virtio disk.  */
       if ((strncmp ("hd", p, 2) == 0
+	   || strncmp ("vd", p, 2) == 0
 	   || strncmp ("sd", p, 2) == 0)
 	  && p[2] >= 'a' && p[2] <= 'z')
 	{
-	  /* /dev/[hs]d[a-z][0-9]* */
+	  /* /dev/[hsv]d[a-z][0-9]* */
 	  p[3] = '\0';
 	  return path;
 	}
+
+      /* If this is a Xen virtual block device.  */
+      if ((strncmp ("xvd", p, 3) == 0) && p[3] >= 'a' && p[3] <= 'z')
+	{
+	  /* /dev/xvd[a-z][0-9]* */
+	  p[4] = '\0';
+	  return path;
+	}
     }
 
   return path;
   
 #elif defined(__GNU__)
-  path = xstrdup (os_dev);
+  char *path = xstrdup (os_dev);
   if (strncmp ("/dev/sd", path, 7) == 0 || strncmp ("/dev/hd", path, 7) == 0)
     {
-      p = strchr (path, 's');
+      char *p = strchr (path + 7, 's');
       if (p)
 	*p = '\0';
     }
   return path;
 
+#elif defined(__CYGWIN__)
+  char *path = xstrdup (os_dev);
+  if (strncmp ("/dev/sd", path, 7) == 0 && 'a' <= path[7] && path[7] <= 'z')
+    path[8] = 0;
+  return path;
+
 #else
-# warning "The function `get_os_disk' might not work on your OS correctly."
+# warning "The function `convert_system_partition_to_system_disk' might not work on your OS correctly."
   return xstrdup (os_dev);
 #endif
 }
 
 static int
-find_drive (const char *os_dev)
+find_system_device (const char *os_dev)
 {
   int i;
   char *os_disk;
 
-  os_disk = get_os_disk (os_dev);
+  os_disk = convert_system_partition_to_system_disk (os_dev);
   if (! os_disk)
     return -1;
   
   for (i = 0; i < (int) (sizeof (map) / sizeof (map[0])); i++)
-    if (map[i] && strcmp (map[i], os_disk) == 0)
+    if (map[i].device && strcmp (map[i].device, os_disk) == 0)
       {
 	free (os_disk);
 	return i;
@@ -703,7 +753,7 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
       return 0;
     }
 
-  drive = find_drive (os_dev);
+  drive = find_system_device (os_dev);
   if (drive < 0)
     {
       grub_error (GRUB_ERR_BAD_DEVICE,
@@ -714,11 +764,15 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
   if (! S_ISBLK (st.st_mode))
     return make_device_name (drive, -1, -1);
   
-#if defined(__linux__)
+#if defined(__linux__) || defined(__CYGWIN__)
   /* Linux counts partitions uniformly, whether a BSD partition or a DOS
      partition, so mapping them to GRUB devices is not trivial.
      Here, get the start sector of a partition by HDIO_GETGEO, and
-     compare it with each partition GRUB recognizes.  */
+     compare it with each partition GRUB recognizes.
+
+     Cygwin /dev/sdXN emulation uses Windows partition mapping. It
+     does not count the extended partition and missing primary
+     partitions.  Use same method as on Linux here.  */
   {
     char *name;
     grub_disk_t disk;
@@ -732,7 +786,7 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
     int find_partition (grub_disk_t disk __attribute__ ((unused)),
 			const grub_partition_t partition)
       {
- 	struct grub_pc_partition *pcdata = 0;
+ 	struct grub_pc_partition *pcdata = NULL;
 
 	if (strcmp (partition->partmap->name, "pc_partition_map") == 0)
 	  pcdata = partition->data;
@@ -777,7 +831,7 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
       return name;
     
     fd = open (os_dev, O_RDONLY);
-    if (! fd)
+    if (fd == -1)
       {
 	grub_error (GRUB_ERR_BAD_DEVICE, "cannot open `%s'", os_dev);
 	free (name);
