@@ -28,13 +28,9 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <dirent.h>
 
 #include <grub/util/misc.h>
 #include <grub/util/deviceiter.h>
-#include <grub/list.h>
-#include <grub/misc.h>
-#include <grub/emu/misc.h>
 
 #ifdef __linux__
 # if !defined(__GLIBC__) || \
@@ -66,23 +62,12 @@ struct hd_geometry
                  | ((unsigned int) (__dev >> 32) & ~0xfff); \
   })
 # endif /* ! MAJOR */
-# ifndef MINOR
-#  define MINOR(dev)	\
-  ({ \
-     unsigned long long __dev = (dev); \
-     (unsigned) (__dev & 0xff) | ((unsigned int) (__dev >> 12) & ~0xff); \
-  })
-# endif /* ! MINOR */
 # ifndef CDROM_GET_CAPABILITY
 #  define CDROM_GET_CAPABILITY	0x5331	/* get capabilities */
 # endif /* ! CDROM_GET_CAPABILITY */
 # ifndef BLKGETSIZE
 #  define BLKGETSIZE	_IO(0x12,96)	/* return device size */
 # endif /* ! BLKGETSIZE */
-
-#ifdef HAVE_DEVICE_MAPPER
-# include <libdevmapper.h>
-#endif
 #endif /* __linux__ */
 
 /* Use __FreeBSD_kernel__ instead of __FreeBSD__ for compatibility with
@@ -303,12 +288,6 @@ get_dac960_disk_name (char *name, int controller, int drive)
   sprintf (name, "/dev/rd/c%dd%d", controller, drive);
 }
 
-static void
-get_acceleraid_disk_name (char *name, int controller, int drive)
-{
-  sprintf (name, "/dev/rs/c%dd%d", controller, drive);
-}
-
 static void
 get_ataraid_disk_name (char *name, int unit)
 {
@@ -346,37 +325,18 @@ get_xvd_disk_name (char *name, int unit)
 }
 #endif
 
-static struct seen_device
-{
-  struct seen_device *next;
-  const char *name;
-} *seen;
-
-/* Check if DEVICE can be read.  Skip any DEVICE that we have already seen.
-   If an error occurs, return zero, otherwise return non-zero.  */
+/* Check if DEVICE can be read. If an error occurs, return zero,
+   otherwise return non-zero.  */
 static int
-check_device_readable_unique (const char *device)
+check_device (const char *device)
 {
-  char *real_device;
   char buf[512];
   FILE *fp;
-  struct seen_device *seen_elt;
 
   /* If DEVICE is empty, just return error.  */
   if (*device == 0)
     return 0;
 
-  /* Have we seen this device already?  */
-  real_device = canonicalize_file_name (device);
-  if (! real_device)
-    return 0;
-  if (grub_named_list_find (GRUB_AS_NAMED_LIST (seen), real_device))
-    {
-      grub_dprintf ("deviceiter", "Already seen %s (%s)\n",
-		    device, real_device);
-      goto fail;
-    }
-
   fp = fopen (device, "r");
   if (! fp)
     {
@@ -399,7 +359,7 @@ check_device_readable_unique (const char *device)
 	  break;
 	}
       /* Error opening the device.  */
-      goto fail;
+      return 0;
     }
 
   /* Make sure CD-ROMs don't get assigned a BIOS disk number
@@ -407,7 +367,7 @@ check_device_readable_unique (const char *device)
 #ifdef __linux__
 # ifdef CDROM_GET_CAPABILITY
   if (ioctl (fileno (fp), CDROM_GET_CAPABILITY, 0) >= 0)
-    goto fail;
+    return 0;
 # else /* ! CDROM_GET_CAPABILITY */
   /* Check if DEVICE is a CD-ROM drive by the HDIO_GETGEO ioctl.  */
   {
@@ -415,14 +375,14 @@ check_device_readable_unique (const char *device)
     struct stat st;
 
     if (fstat (fileno (fp), &st))
-      goto fail;
+      return 0;
 
     /* If it is a block device and isn't a floppy, check if HDIO_GETGEO
        succeeds.  */
     if (S_ISBLK (st.st_mode)
 	&& MAJOR (st.st_rdev) != FLOPPY_MAJOR
 	&& ioctl (fileno (fp), HDIO_GETGEO, &hdg))
-      goto fail;
+      return 0;
   }
 # endif /* ! CDROM_GET_CAPABILITY */
 #endif /* __linux__ */
@@ -430,7 +390,7 @@ check_device_readable_unique (const char *device)
 #if defined(__FreeBSD_kernel__) || defined(__NetBSD__) || defined(__OpenBSD__)
 # ifdef CDIOCCLRDEBUG
   if (ioctl (fileno (fp), CDIOCCLRDEBUG, 0) >= 0)
-    goto fail;
+    return 0;
 # endif /* CDIOCCLRDEBUG */
 #endif /* __FreeBSD_kernel__ || __NetBSD__ || __OpenBSD__ */
 
@@ -438,73 +398,19 @@ check_device_readable_unique (const char *device)
   if (fread (buf, 1, 512, fp) != 512)
     {
       fclose (fp);
-      goto fail;
+      return 0;
     }
 
-  /* Remember that we've seen this device.  */
-  seen_elt = xmalloc (sizeof (*seen_elt));
-  seen_elt->name = real_device; /* steal memory */
-  grub_list_push (GRUB_AS_LIST_P (&seen), GRUB_AS_LIST (seen_elt));
-
   fclose (fp);
   return 1;
-
-fail:
-  free (real_device);
-  return 0;
 }
 
-static void
-clear_seen_devices (void)
-{
-  while (seen)
-    {
-      struct seen_device *seen_elt = seen;
-      seen = seen->next;
-      free (seen_elt);
-    }
-  seen = NULL;
-}
-
-#ifdef __linux__
-struct device
-{
-	char *stable;
-	char *kernel;
-};
-
-/* Sort by the kernel name for preference since that most closely matches
-   older device.map files, but sort by stable by-id names as a fallback.
-   This is because /dev/disk/by-id/ usually has a few alternative
-   identifications of devices (e.g. ATA vs. SATA).
-   check_device_readable_unique will ensure that we only get one for any
-   given disk, but sort the list so that the choice of which one we get is
-   stable.  */
-static int
-compare_devices (const void *a, const void *b)
-{
-  const struct device *left = (const struct device *) a;
-  const struct device *right = (const struct device *) b;
-
-  if (left->kernel && right->kernel)
-    {
-      int ret = strcmp (left->kernel, right->kernel);
-      if (ret)
-	return ret;
-    }
-
-  return strcmp (left->stable, right->stable);
-}
-#endif /* __linux__ */
-
 void
 grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 			   int floppy_disks)
 {
   int i;
 
-  clear_seen_devices ();
-
   /* Floppies.  */
   for (i = 0; i < floppy_disks; i++)
     {
@@ -514,73 +420,13 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
       get_floppy_disk_name (name, i);
       if (stat (name, &st) < 0)
 	break;
-      /* In floppies, write the map, whether check_device_readable_unique
-         succeeds or not, because the user just may not insert floppies.  */
+      /* In floppies, write the map, whether check_device succeeds
+	 or not, because the user just may not insert floppies.  */
       if (hook (name, 1))
-	goto out;
+	return;
     }
 
 #ifdef __linux__
-  {
-    DIR *dir = opendir ("/dev/disk/by-id");
-
-    if (dir)
-      {
-	struct dirent *entry;
-	struct device *devs;
-	size_t devs_len = 0, devs_max = 1024, i;
-
-	devs = xmalloc (devs_max * sizeof (*devs));
-
-	/* Dump all the directory entries into names, resizing if
-	   necessary.  */
-	for (entry = readdir (dir); entry; entry = readdir (dir))
-	  {
-	    /* Skip current and parent directory entries.  */
-	    if (strcmp (entry->d_name, ".") == 0 ||
-		strcmp (entry->d_name, "..") == 0)
-	      continue;
-	    /* Skip partition entries.  */
-	    if (strstr (entry->d_name, "-part"))
-	      continue;
-	    /* Skip device-mapper entries; we'll handle the ones we want
-	       later.  */
-	    if (strncmp (entry->d_name, "dm-", sizeof ("dm-") - 1) == 0)
-	      continue;
-	    /* Skip RAID entries; they are handled by upper layers.  */
-	    if (strncmp (entry->d_name, "md-", sizeof ("md-") - 1) == 0)
-	      continue;
-	    if (devs_len >= devs_max)
-	      {
-		devs_max *= 2;
-		devs = xrealloc (devs, devs_max * sizeof (*devs));
-	      }
-	    devs[devs_len].stable =
-	      xasprintf ("/dev/disk/by-id/%s", entry->d_name);
-	    devs[devs_len].kernel =
-	      canonicalize_file_name (devs[devs_len].stable);
-	    devs_len++;
-	  }
-
-	qsort (devs, devs_len, sizeof (*devs), &compare_devices);
-
-	closedir (dir);
-
-	/* Now add all the devices in sorted order.  */
-	for (i = 0; i < devs_len; ++i)
-	  {
-	    if (check_device_readable_unique (devs[i].stable))
-	      {
-		if (hook (devs[i].stable, 0))
-		  goto out;
-	      }
-	    free (devs[i].stable);
-	    free (devs[i].kernel);
-	  }
-	free (devs);
-      }
-  }
-
   if (have_devfs ())
     {
       i = 0;
@@ -600,23 +446,23 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 	    {
 	      strcat (name, "/disc");
 	      if (hook (name, 0))
-		goto out;
+		return;
 	    }
 	}
-      goto out;
+      return;
     }
 #endif /* __linux__ */
 
   /* IDE disks.  */
-  for (i = 0; i < 96; i++)
+  for (i = 0; i < 26; i++)
     {
       char name[16];
 
       get_ide_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
 	}
     }
 
@@ -627,10 +473,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
       char name[16];
 
       get_virtio_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
 	}
     }
 
@@ -640,10 +486,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
       char name[20];
 
       get_ataraid_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
         }
     }
 
@@ -653,24 +499,24 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
       char name[16];
 
       get_xvd_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
 	}
     }
 #endif /* __linux__ */
 
   /* The rest is SCSI disks.  */
-  for (i = 0; i < 48; i++)
+  for (i = 0; i < 26; i++)
     {
       char name[16];
 
       get_scsi_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
 	}
     }
 
@@ -690,31 +536,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 	    char name[24];
 
 	    get_dac960_disk_name (name, controller, drive);
-	    if (check_device_readable_unique (name))
+	    if (check_device (name))
 	      {
 		if (hook (name, 0))
-		  goto out;
-	      }
-	  }
-      }
-  }
-
-  /* This is for Mylex Acceleraid - we have
-     /dev/rd/c<controller>d<logical drive>p<partition>.  */
-  {
-    int controller, drive;
-
-    for (controller = 0; controller < 8; controller++)
-      {
-	for (drive = 0; drive < 15; drive++)
-	  {
-	    char name[24];
-
-	    get_acceleraid_disk_name (name, controller, drive);
-	    if (check_device_readable_unique (name))
-	      {
-		if (hook (name, 0))
-		  goto out;
+		  return;
 	      }
 	  }
       }
@@ -732,10 +557,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 	    char name[24];
 
 	    get_cciss_disk_name (name, controller, drive);
-	    if (check_device_readable_unique (name))
+	    if (check_device (name))
 	      {
 		if (hook (name, 0))
-		  goto out;
+		  return;
 	      }
 	  }
       }
@@ -753,10 +578,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 	    char name[24];
 
 	    get_ida_disk_name (name, controller, drive);
-	    if (check_device_readable_unique (name))
+	    if (check_device (name))
 	      {
 		if (hook (name, 0))
-		  goto out;
+		  return;
 	      }
 	  }
       }
@@ -771,10 +596,10 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
 	char name[24];
 
 	get_i2o_disk_name (name, unit);
-	if (check_device_readable_unique (name))
+	if (check_device (name))
 	  {
 	    if (hook (name, 0))
-	      goto out;
+	      return;
 	  }
       }
   }
@@ -785,107 +610,12 @@ grub_util_iterate_devices (int NESTED_FUNC_ATTR (*hook) (const char *, int),
       char name[16];
 
       get_mmc_disk_name (name, i);
-      if (check_device_readable_unique (name))
+      if (check_device (name))
 	{
 	  if (hook (name, 0))
-	    goto out;
+	    return;
 	}
     }
-
-# ifdef HAVE_DEVICE_MAPPER
-#  define dmraid_check(cond, ...) \
-  if (! (cond)) \
-    { \
-      grub_dprintf ("deviceiter", __VA_ARGS__); \
-      goto dmraid_end; \
-    }
-
-  /* DM-RAID.  */
-  if (grub_device_mapper_supported ())
-    {
-      struct dm_tree *tree = NULL;
-      struct dm_task *task = NULL;
-      struct dm_names *names = NULL;
-      unsigned int next = 0;
-      void *top_handle, *second_handle;
-      struct dm_tree_node *root, *top, *second;
-
-      /* Build DM tree for all devices.  */
-      tree = dm_tree_create ();
-      dmraid_check (tree, "dm_tree_create failed\n");
-      task = dm_task_create (DM_DEVICE_LIST);
-      dmraid_check (task, "dm_task_create failed\n");
-      dmraid_check (dm_task_run (task), "dm_task_run failed\n");
-      names = dm_task_get_names (task);
-      dmraid_check (names, "dm_task_get_names failed\n");
-      dmraid_check (names->dev, "No DM devices found\n");
-      do
-	{
-	  names = (void *) names + next;
-	  dmraid_check (dm_tree_add_dev (tree, MAJOR (names->dev),
-					 MINOR (names->dev)),
-			   "dm_tree_add_dev (%s) failed\n", names->name);
-	  next = names->next;
-	}
-      while (next);
-
-      /* Walk the second-level children of the inverted tree; that is, devices
-	 which are directly composed of non-DM devices such as hard disks.
-	 This class includes all DM-RAID disks and excludes all DM-RAID
-	 partitions.  */
-      root = dm_tree_find_node (tree, 0, 0);
-      top_handle = NULL;
-      top = dm_tree_next_child (&top_handle, root, 1);
-      while (top)
-	{
-	  second_handle = NULL;
-	  second = dm_tree_next_child (&second_handle, top, 1);
-	  while (second)
-	    {
-	      const char *node_name, *node_uuid;
-	      char *name;
-
-	      node_name = dm_tree_node_get_name (second);
-	      dmraid_check (node_name, "dm_tree_node_get_name failed\n");
-	      node_uuid = dm_tree_node_get_uuid (second);
-	      dmraid_check (node_uuid, "dm_tree_node_get_uuid failed\n");
-	      if (strncmp (node_uuid, "DMRAID-", 7) != 0)
-		{
-		  grub_dprintf ("deviceiter", "%s is not DM-RAID\n", node_name);
-		  goto dmraid_next_child;
-		}
-
-	      name = xasprintf ("/dev/mapper/%s", node_name);
-	      if (check_device_readable_unique (name))
-		{
-		  if (hook (name, 0))
-		    {
-		      free (name);
-		      if (task)
-			dm_task_destroy (task);
-		      if (tree)
-			dm_tree_free (tree);
-		      goto out;
-		    }
-		}
-	      free (name);
-
-dmraid_next_child:
-	      second = dm_tree_next_child (&second_handle, top, 1);
-	    }
-	  top = dm_tree_next_child (&top_handle, root, 1);
-	}
-
-dmraid_end:
-      if (task)
-	dm_task_destroy (task);
-      if (tree)
-	dm_tree_free (tree);
-    }
-# endif /* HAVE_DEVICE_MAPPER */
 #endif /* __linux__ */
-
-out:
-  clear_seen_devices ();
 }
 
