@@ -1,4 +1,4 @@
-/* hostdisk.c - emulate biosdisk */
+/* biosdisk.c - emulate biosdisk */
 /*
  *  GRUB  --  GRand Unified Bootloader
  *  Copyright (C) 1999,2000,2001,2002,2003,2004,2006,2007,2008,2009  Free Software Foundation, Inc.
@@ -19,13 +19,12 @@
 
 #include <grub/disk.h>
 #include <grub/partition.h>
-#include <grub/msdos_partition.h>
+#include <grub/pc_partition.h>
 #include <grub/types.h>
 #include <grub/err.h>
 #include <grub/util/misc.h>
 #include <grub/util/hostdisk.h>
 #include <grub/misc.h>
-#include <grub/i18n.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -128,7 +127,7 @@ find_grub_drive (const char *name)
 
   if (name)
     {
-      for (i = 0; i < ARRAY_SIZE (map); i++)
+      for (i = 0; i < sizeof (map) / sizeof (map[0]); i++)
 	if (map[i].drive && ! strcmp (map[i].drive, name))
 	  return i;
     }
@@ -137,7 +136,7 @@ find_grub_drive (const char *name)
 }
 
 static int
-find_free_slot (void)
+find_free_slot ()
 {
   unsigned int i;
 
@@ -336,12 +335,11 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
     char dev[PATH_MAX];
 
     strcpy (dev, map[disk->id].device);
-    if (disk->partition && sector >= disk->partition->start
-	&& strncmp (map[disk->id].device, "/dev/", 5) == 0)
+    if (disk->partition && strncmp (map[disk->id].device, "/dev/", 5) == 0)
       is_partition = linux_find_partition (dev, disk->partition->start);
 
     /* Open the partition.  */
-    grub_dprintf ("hostdisk", "opening the device `%s' in open_device()\n", dev);
+    grub_dprintf ("hostdisk", "opening the device `%s' in open_device()", dev);
     fd = open (dev, flags);
     if (fd < 0)
       {
@@ -349,8 +347,7 @@ open_device (const grub_disk_t disk, grub_disk_addr_t sector, int flags)
 	return -1;
       }
 
-    /* Flush the buffer cache to the physical disk.
-       XXX: This also empties the buffer cache.  */
+    /* Make the buffer cache consistent with the physical disk.  */
     ioctl (fd, BLKFLSBUF, 0);
 
     if (is_partition)
@@ -491,23 +488,6 @@ grub_util_biosdisk_read (grub_disk_t disk, grub_disk_addr_t sector,
 {
   int fd;
 
-  /* Split pre-partition and partition reads.  */
-  if (disk->partition && sector < disk->partition->start
-      && sector + size > disk->partition->start)
-    {
-      grub_err_t err;
-      err = grub_util_biosdisk_read (disk, sector,
-				     disk->partition->start - sector,
-				     buf);
-      if (err)
-	return err;
-
-      return grub_util_biosdisk_read (disk, disk->partition->start,
-				      size - (disk->partition->start - sector),
-				      buf + ((disk->partition->start - sector)
-					     << GRUB_DISK_SECTOR_BITS));
-    }
-
   fd = open_device (disk, sector, O_RDONLY);
   if (fd < 0)
     return grub_errno;
@@ -545,23 +525,6 @@ grub_util_biosdisk_write (grub_disk_t disk, grub_disk_addr_t sector,
 {
   int fd;
 
-  /* Split pre-partition and partition writes.  */
-  if (disk->partition && sector < disk->partition->start
-      && sector + size > disk->partition->start)
-    {
-      grub_err_t err;
-      err = grub_util_biosdisk_write (disk, sector,
-				      disk->partition->start - sector,
-				      buf);
-      if (err)
-	return err;
-
-      return grub_util_biosdisk_write (disk, disk->partition->start,
-				       size - (disk->partition->start - sector),
-				       buf + ((disk->partition->start - sector)
-					      << GRUB_DISK_SECTOR_BITS));
-    }
-
   fd = open_device (disk, sector, O_WRONLY);
   if (fd < 0)
     return grub_errno;
@@ -602,10 +565,7 @@ read_device_map (const char *dev_map)
 
   fp = fopen (dev_map, "r");
   if (! fp)
-    {
-      grub_util_info (_("cannot open `%s'"), dev_map);
-      return;
-    }
+    grub_util_error ("Cannot open `%s'", dev_map);
 
   while (fgets (buf, sizeof (buf), fp))
     {
@@ -674,7 +634,7 @@ read_device_map (const char *dev_map)
 	 symbolic links.  */
       map[drive].device = xmalloc (PATH_MAX);
       if (! realpath (p, map[drive].device))
-	grub_util_error ("cannot get the real path of `%s'", p);
+	grub_util_error ("Cannot get the real path of `%s'", p);
 #else
       map[drive].device = xstrdup (p);
 #endif
@@ -710,27 +670,34 @@ grub_util_biosdisk_fini (void)
 static char *
 make_device_name (int drive, int dos_part, int bsd_part)
 {
-  char *ret;
-  char *dos_part_str = NULL;
-  char *bsd_part_str = NULL;
+  int len = strlen(map[drive].drive);
+  char *p;
 
   if (dos_part >= 0)
-    dos_part_str = xasprintf (",%d", dos_part + 1);
+    {
+      /* Add in char length of dos_part+1 */
+      int tmp = dos_part + 1;
+      len++;
+      while ((tmp /= 10) != 0)
+	len++;
+    }
+  if (bsd_part >= 0)
+    len += 2;
+
+  /* Length to alloc is: char length of map[drive].drive, plus
+   *                     char length of (dos_part+1) or of bsd_part, plus
+   *                     2 for the comma and a null/end of string (\0)
+   */
+  p = xmalloc (len + 2);
+  sprintf (p, "%s", map[drive].drive);
+
+  if (dos_part >= 0)
+    sprintf (p + strlen (p), ",%d", dos_part + 1);
 
   if (bsd_part >= 0)
-    bsd_part_str = xasprintf (",%c", dos_part + 'a');
+    sprintf (p + strlen (p), ",%c", bsd_part + 'a');
 
-  ret = xasprintf ("%s%s%s", map[drive].drive,
-                   dos_part_str ? : "",
-                   bsd_part_str ? : "");
-
-  if (dos_part_str)
-    free (dos_part_str);
-
-  if (bsd_part_str)
-    free (bsd_part_str);
-
-  return ret;
+  return p;
 }
 
 static char *
@@ -739,7 +706,7 @@ convert_system_partition_to_system_disk (const char *os_dev)
 #if defined(__linux__)
   char *path = xmalloc (PATH_MAX);
   if (! realpath (os_dev, path))
-    return NULL;
+    return 0;
 
   if (strncmp ("/dev/", path, 5) == 0)
     {
@@ -776,16 +743,6 @@ convert_system_partition_to_system_disk (const char *os_dev)
 	  return path;
 	}
 
-      /* If this is a Mylex AcceleRAID Array.  */
-      if (strncmp ("rs/c", p, 4) == 0)
-	{
-	  /* /dev/rd/c[0-9]+d[0-9]+(p[0-9]+)? */
-	  p = strchr (p, 'p');
-	  if (p)
-	    *p = '\0';
-
-	  return path;
-	}
       /* If this is a CCISS disk.  */
       if (strncmp ("cciss/c", p, sizeof ("cciss/c") - 1) == 0)
 	{
@@ -909,29 +866,22 @@ device_is_wholedisk (const char *os_dev)
 static int
 find_system_device (const char *os_dev)
 {
-  unsigned int i;
+  int i;
   char *os_disk;
 
   os_disk = convert_system_partition_to_system_disk (os_dev);
   if (! os_disk)
     return -1;
 
-  for (i = 0; i < ARRAY_SIZE (map); i++)
-    if (! map[i].device)
-      break;
-    else if (strcmp (map[i].device, os_disk) == 0)
+  for (i = 0; i < (int) (sizeof (map) / sizeof (map[0])); i++)
+    if (map[i].device && strcmp (map[i].device, os_disk) == 0)
       {
 	free (os_disk);
 	return i;
       }
 
-  if (i == ARRAY_SIZE (map))
-    grub_util_error (_("device count exceeds limit"));
-
-  map[i].device = os_disk;
-  map[i].drive = xstrdup (os_disk);
-
-  return i;
+  free (os_disk);
+  return -1;
 }
 
 char *
@@ -981,15 +931,15 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
     struct hd_geometry hdg;
     int dos_part = -1;
     int bsd_part = -1;
-    auto int find_partition (grub_disk_t dsk,
+    auto int find_partition (grub_disk_t disk,
 			     const grub_partition_t partition);
 
-    int find_partition (grub_disk_t dsk __attribute__ ((unused)),
+    int find_partition (grub_disk_t disk __attribute__ ((unused)),
 			const grub_partition_t partition)
       {
- 	struct grub_msdos_partition *pcdata = NULL;
+ 	struct grub_pc_partition *pcdata = NULL;
 
-	if (strcmp (partition->partmap->name, "part_msdos") == 0)
+	if (strcmp (partition->partmap->name, "pc_partition_map") == 0)
 	  pcdata = partition->data;
 
 	if (pcdata)
@@ -1097,7 +1047,7 @@ grub_util_biosdisk_get_grub_dev (const char *os_dev)
 	n = strtol (p, &q, 10);
 	if (p != q && n != GRUB_LONG_MIN && n != GRUB_LONG_MAX)
 	  {
-	    dos_part = (int) n - 1;
+	    dos_part = (int) n;
 
 	    if (*q >= 'a' && *q <= 'g')
 	      bsd_part = *q - 'a';
